#!/usr/bin/env python

import unittest, re
from __init__ import Technorati
from lxml import etree
from datetime import date, timedelta
import time

TECHNORATI_TEST_TAGS = ['linux', ('debian', 'linux') ]

class TestTechnorati(unittest.TestCase):

    def test_tag_info(self):
        print '### Testing tag_info ###'

        countTags = 0

        for tags in TECHNORATI_TEST_TAGS:
            countTags = Technorati.getTagInfo(tags)
            print '%s has %s counts ' % (tags, countTags)

            assert countTags > 0



    def test_related_tags(self):
        print '### Testing related_tags ###'

        print 'not supported by Technorati at the moment'

#        countTags = 0
#        
#        for tags in TECHNORATI_TEST_TAGS:
#            countTags = Technorati.getRelatedTags( tags )
#            print '%s has related tags: %s' % (tags, countTags)
#        assert countTags > 0

    def test_parse_url(self):

        print '### Test parsing URL ###'
        assert Technorati._parseURL('linux') == 'http://technorati.com/search?usingAdvanced=1&q=linux&return=posts&source=advanced-source-all&topic=overall&authority=high'
        assert Technorati._parseURL(('debian', 'linux')) == 'http://technorati.com/search?usingAdvanced=1&q=debian+linux&return=posts&source=advanced-source-all&topic=overall&authority=high'

    def test_get_blog_links(self):
        ''' tests retrieving the blog links '''

        links = Technorati.get_blog_links('climate change', maxResults=10)
        assert len(links) > 0

        links = Technorati.get_blog_links(['climate', 'change'], 20)
        assert len(links) == 20

        links = Technorati.get_blog_links(['climate', 'change'], 33)
        assert len(links) == 33

    def test_max_age(self):
        ''' tests fetching the articles by maxAge '''

        def testMaxAge(age, dimension, maxAge=0):
            if dimension == 'weeks': hours = age * 7 * 24
            elif dimension == 'days': hours = age * 24
            elif dimension == 'hours': hours = age
            elif dimension == 'minutes': hours = 1
            else: hours = age

            root = etree.Element("root")
            div = etree.SubElement(root, "div")
            div.text = '{age} {dimension} ago'.format(age=age, dimension=dimension)
            print 'Testing %s' % div.text
            linkDate = Technorati._getDate(root, 0)
            print 'LinkDate = ', linkDate
            return (date.fromtimestamp(time.time()) - timedelta(hours=hours)) == Technorati._getDate(root, maxAge)

        assert testMaxAge(7, 'weeks')
        assert testMaxAge(12, 'hours')
        assert testMaxAge(23, 'minutes')
        assert testMaxAge(8, 'minutes', 1)
        assert False == testMaxAge(23, 'miasdfasdnutes')
        assert False == testMaxAge(8, 'weeks', 5)
        assert False == testMaxAge(8, 'days', 5)

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 25.09.2012

@author: Norman Süsstrunk, Heinz-Peter Lang, Albert Weichselbraun
'''
import unittest
import logging
from operator import attrgetter
from datetime import datetime, timedelta
from cPickle import dump

from gdata.youtube.service import YouTubeService, YouTubeVideoQuery

from eWRT.ws.WebDataSource import WebDataSource
from time import sleep
import dateutil.parser as dateparser
from types import NoneType

from apiclient.discovery import build
import urllib, json

def get_value(key, dictionary):
    if '.' in key and not key.isdigit():
        old_key, new_key = key.split('.', 1)
        if old_key in dictionary:
            return get_value(new_key, dictionary[old_key])
    else: 
        if key in dictionary:
            return dictionary[key]
    
    return None

# number of seconds to wait between comment request
# this is required to prevent the youtube api from
# blocking us.
YOUTUBE_SLEEP_TIME = 1       
MAX_RESULTS_PER_QUERY = 50 
DATE_FORMAT = '%Y-%m-%dT%H:%M:%S.000Z'

logger = logging.getLogger('eWRT.ws.youtube')
convert_date = lambda str_date: datetime.strptime(str_date, DATE_FORMAT)

# ToDO: comment rating, once it get's supported by the gdata API.
# TODO: query.location --> radius available?

FREEBASE_SEARCH_URL = 'https://www.googleapis.com/freebase/v1/search?%s'
YOUTUBE_SEARCH_URL = 'https://www.youtube.com/watch?v='

class YouTubeEntry(dict):
    
    DATE_FIELDS = ('published', 'last_modified')
     
    VIDEO_MAPPING = {
        'id':'video_id',
        'comments': None,
        'snippet.title': 'title',
        'snippet.publishedAt':'published',
        'snippet.description': 'content',
        'contentDetails.duration':'duration',
        'contentDetails.caption':'caption',
        'contentDetails.licensedContent':'licensed',
        'statistics.viewCount':'statistics_viewcount',
        'statistics.favoriteCount':'statistics_favoritecount',
        'statistics.likeCount':'statistics_likecount',
        'statistics.dislikeCount':'statistics_dislikecount',
        'statistics.commentCount':'statistics_commentcount',
        'topicDetails.relevantTopicIds':'freebase_topics_relevant',
        'topicDetails.topicIds':'freebase_topics'     
    }
    
    COMMENT_MAPPING = {
        'id': 'comment_id',
        'parent':'parent_id',
        'snippet.authorDisplayName':'user_name',
        'snippet.authorGoogleplusProfileUrl':'user_profile',
        'snippet.viewer_rating':'viewer_rating',
        'snippet.likeCount':'like_count',
        'snippet.totalReplyCount':'reply_count',
        'snippet.channelId':'channel_id',
        'snippet.publishedAt':'published',
        'snippet.updatedAt':'last_modified',
        'snippet.textDisplay':'content',
    }
    
    def __init__(self, search_result, mapping=VIDEO_MAPPING):
        ''' constructor initializes entry with the result'''
        dict.__init__(self)
        self.update(self.update_entry(search_result, mapping=mapping))
    
        if 'video_id' in self:
            self['url'] = ''.join([YOUTUBE_SEARCH_URL, self['video_id']])
        
    def __repr__(self):
        return '** entry: %s' % '\n'.join(['%s: %s' % (k, v) for k, v in self.iteritems()])

    def update_entry(self, search_result, mapping=VIDEO_MAPPING):
        ''' stores the mapped items in a dictionary 
        @param search_result: the search result returned by youtube
        @param entry_type: string, either comment or video
        '''             
        for attr, key in mapping.iteritems(): 
            
            if attr == 'comments' and 'comments' in search_result:
                self['comments'] = search_result['comments']
                continue
            
            value = get_value(attr, search_result)
            if isinstance(value, unicode):
                value = value.encode('utf-8')
            if key in self and isinstance(self[key], list):
                if value:
                    self[key].append(value)
            else: 
                self[key] = value
               
        #parse dates 
        if 'published' in self and not 'last_modified' in self:
            self['last_modified'] = self['published']

        for date_field in self.DATE_FIELDS:
            if isinstance(self[date_field], basestring):
                self[date_field] = dateparser.parse(self[date_field],
                                                    ignoretz=True)
        return self
    

class YouTube_v3(WebDataSource):
    
    YOUTUBE_API_SERVICE_NAME = 'youtube'
    YOUTUBE_API_VERSION = 'v3'

    def __init__(self, api_key):
        self.api_key = api_key
        self.client = build(self.YOUTUBE_API_SERVICE_NAME, 
                            self.YOUTUBE_API_VERSION,
                            developerKey=self.api_key)

    @classmethod
    def _get_yt_dict(cls, entry, mapping):
        """ stores the mapped items in a dictionary 
        @param entry: Gdata Entry
        @param mapping: dictionary with the mapping
        @return: yt_dict
        """
        yt_dict = {}
        for attr, key in mapping.iteritems(): 
            try: 
                yt_dict[key] = attrgetter(attr)(entry)
                
                if key in ('published', 'last_modified'):
                    yt_dict[key] = convert_date(yt_dict[key])
                
            except AttributeError, e:
                logger.warn('AttributeError: %s' % e)
                yt_dict[key] = None        
        
        return yt_dict
    
    def get_video_search_feed(self, search_terms, max_results=25, max_comment_count=0):
        """
        Returns a generator of youtube search results
        """
        for search_result in self.search(search_terms, max_results):
            if search_result['id']['kind'] == 'youtube#video':
                try:
                    yield self._convert_item_to_video(search_result, 
                                                      max_comment_count)
                except Exception, e:
                    logger.error('Failed to convert Youtube search result: %s' % e)
                
    def _get_video_rating(self, video_id):
        """ Returns the rating for a video ID """
        return self.client.videos().getRating(id=video_id).execute()
        
    def _get_video_details(self, video_id):
        return self.client.videos().list(id=video_id, 
                                         part='contentDetails,statistics,topicDetails').execute()
    
    def like_video(self, video_id):
        """ Adds to the video rating. This code sets the rating to "like," but you 
            could also support an additional option that supports values of "like"
            and "dislike"."""
        self.client.videos().rate(id=video_id, rating="like").execute()

    def _build_youtube_item(self, item, max_comment_count=0, get_details=False):
        """ """
        
        video_id = item['id']['videoId']
        
        # retrieve comments and details as requested
        if max_comment_count > 0:
            item['comments'] = [YouTubeEntry(comment, YouTubeEntry.COMMENT_MAPPING) 
                                for comment in self._get_video_comments(video_id=video_id)]
 
        if get_details: 
            details = self._get_video_details(video_id=video_id)['items']
            if details and len(details)>0:
                item.update(details[0])

        return YouTubeEntry(item)
    
    def search(self, 
               search_terms, 
               max_results=MAX_RESULTS_PER_QUERY, 
               since_date=None, 
               region_code=None, 
               language=None):
        """ 
        Search the youtube API for videos matching a set of search terms 
        @param search_terms, a comma separated list of search terms
        @param max_results, the maximum number of results returned
        @param max_age, the maximum number of days ago from which to include results
        @param region_code, ISO 3166-1 alpha-2 country code, e.g. AT, DE, GB, US
        @param language, ISO 639-1 two-letter language code, e.g. de, cs, en
        """
        if not since_date:
            since_date = datetime.now() - timedelta(days=1)
        if isinstance(since_date, datetime):
            since_date = since_date.isoformat("T") + "Z"
            
        items_per_page = min([max_results, MAX_RESULTS_PER_QUERY])
            
        kwargs = {'q':search_terms,
                  'part':'id,snippet',
                  'type':'video',
                  'publishedAfter':since_date,
                  'maxResults':items_per_page,
                  'order':'date'}
        
        if region_code:
            kwargs['regionCode'] = region_code
        if language:
            kwargs['relevanceLanguage'] = language
                                         
        continue_search = True
        items_count = 0 
        
        while continue_search: 
        
            response = self.client.search().list(**kwargs).execute()
            total_results = response['pageInfo']['totalResults']
            
            for search_result in response.get('items', []):
                if search_result['id']['kind'] == 'youtube#video':
                    try:
                        items_count += 1
                        yield self._build_youtube_item(search_result,
                                                       max_comment_count=10, 
                                                       get_details=True)
                    except Exception,e:
                        logger.error('Failed to convert Youtube item: %s' %e) 
            
            if items_count >= max_results:
                continue_search = False
            
            if items_count >= total_results:
                continue_search = False 
                
            if not 'nextPageToken' in response:
                continue_search = False
            else:
                kwargs['pageToken'] = response['nextPageToken']
                 
    def _get_video_comments(self, video_id):
        """ Returns the comments for a youtube ID"""
        comments = self.client.commentThreads().list(part='snippet',
                                                     videoId=video_id,
                                                     textFormat='plainText'
                                                     ).execute()
        result = []
        for item in comments['items']:
            result.append(item['snippet']['topLevelComment'])

        return result
    
    def get_freebase_topics(self, QUERY_TERM):
        """ Retrieves a list of Freebase topics associated with the query term """
        freebase_params = dict(query=QUERY_TERM, key=self.api_key)
        freebase_url = FREEBASE_SEARCH_URL % urllib.urlencode(freebase_params)
        freebase_response = json.loads(urllib.urlopen(freebase_url).read())
        if len(freebase_response['result']) == 0:
            print('No matching terms were found in Freebase.')
            
        return freebase_response['result']

class YouTube(WebDataSource):
    '''
    searches youtube video library
    '''
    
    YT_ATOM_RESULT_TO_DICT_MAPPING = {
        'media.title.text': 'title',
        'published.text':'published',
        'media.description.text': 'content',
        'media.duration.seconds':'duration', 
        'statistics.view_count': 'statistics_viewcount', 
        'statistics.favorite_count': 'statistics_favoritecount', 
        'rating.average': 'rating_average',
        'rating.max': 'rating_max',
        'rating.min': 'rating_min',
        'rating.num_raters': 'rating_numraters', 
        'summary': 'summary', 
        'rights': 'rights', 
        'updated.text': 'last_modified', 
        'source': 'yt_source'
    }
    
    YT_COMMENTS_MAPPING = {
        'id.text': 'id', 
        'title.text': 'title',
        'published.text': 'published',
        'updated.text': 'last_modified',
        'content.text': 'content'
    }
    
    def __init__(self):
        WebDataSource.__init__(self)
        self.youtube_service = YouTubeService()

        
    def search(self, search_terms, location=None, 
               max_results=MAX_RESULTS_PER_QUERY, max_age=None,
               orderby= 'published',
               max_comment_count=0):
        """ 
        Searches for youtube videos.
        
        @param search_terms: list of search terms
        @param location: tuple latitude, longitue, e.g. 37.42307,-122.08427
        @param max_results:
        @param max_age: datetime of the oldest entry  
        @param orderby: order search results by (relevance, published, 
                        viewCount, rating)
        @param max_comment_count: maximum number of comments to fetch 
                                  (default: 0)
        """
        
        if not (isinstance(search_terms, list) or 
            isinstance(search_terms, tuple) or isinstance(search_terms, set)):
            raise ValueError("Warning search requires a list of search terms, \
                             rather than a single term")

        # all youtube search parameter are here: 
        # https://developers.google.com/youtube/2.0/reference?hl=de#Custom_parameters
        query = YouTubeVideoQuery()
        query.vq = ', '.join(search_terms)
        query.orderby = orderby
        query.racy = 'include'
        query.time = self.get_query_time(max_age)
        query.max_results = MAX_RESULTS_PER_QUERY

        if location:
            query.location = location
                        
        return self.search_youtube(query, max_results, max_comment_count)
    

    @classmethod
    def get_query_time(cls, max_age):
        ''' converts a datetime or int (age in minutes) to the youtube specific
        query parameter (e.g. this_month, today ...)
        @param max_age: int or datetime object
        @return: youtube specific query_time 
        '''
        if not max_age:
            return 'all_time'
        
        if isinstance(max_age, datetime):
            # convert datetime to minutes
            max_age = (datetime.now() - max_age).total_seconds() / 60  

        if max_age <= 1440:
            query_time = 'today'
        elif max_age > 1440 and max_age <= 10080:
            query_time = 'this_week'
        else: 
            query_time = 'this_month'
            
        return query_time
  
    def search_youtube(self, query, max_results=MAX_RESULTS_PER_QUERY, 
                       max_comment_count=0):
        ''' executes the youtube query and facilitates paging of the resultset
        @param query: YouTubeVideoQuery
        @param max_results: 
        @param max_comment_count: maximum number of comments to fetch
        @return: list of dictionaries 
        '''
        result = []
        feed = self.youtube_service.YouTubeQuery(query)
        
        while feed:
            for entry in feed.entry: 
                try: 
                    yt_dict = self.convert_feed_entry(entry, max_comment_count)
                    result.append(yt_dict)
                except Exception, e: 
                    logger.exception('Exception converting entry: %s' % e)
                    
                if len(result) == max_results:
                    return result
                
            if not feed.GetNextLink():
                break
            
            feed = self.youtube_service.GetYouTubeVideoFeed(
                                                    feed.GetNextLink().href)
            
        return result


    def convert_feed_entry(self, entry, max_comment_count):
        ''' 1. converts the feed entry to a dictionary (never change the mapping
               names, as later analyzer steps requires consistent keys)
            2. fetches comments and integrate them in the dictionary, if 
               necessary
            
            @param entry: Youtube feed entry
            @param max_comment_count: maximum number of comments to fetch
            @return: dictionary   
        '''

        yt_dict = self._get_yt_dict(entry, self.YT_ATOM_RESULT_TO_DICT_MAPPING)
        yt_dict.update({'user_name': entry.author[0].name.text, 
                        'user_url': entry.author[0].uri.text,
                        'id': entry.id.text.split('/')[-1]})
                       
        yt_dict['url'] = "http://www.youtube.com/watch?v=%s" % yt_dict['id']         
        
        # the hasattr is required for compatibility with older videos
        yt_dict['location'] = entry.geo.location() \
            if hasattr(entry, 'geo') and entry.geo else None
        
        yt_dict['related_url'] = None
        for link in entry.link:
            if link.href.endswith('related'):
                yt_dict['related_url'] = link.href
        
        #mcg: yt APIv3 returns ISO8601, do not format
        if yt_dict['duration']: 
            duration = yt_dict['duration']
        
        yt_dict['picture'] = None

        if hasattr(entry, 'media'):
            for thumbnail in entry.media.thumbnail:
                yt_dict['picture'] = thumbnail.url
                break

        yt_dict['keywords'] = [ category.label for category in entry.category \
              if category.scheme != "http://schemas.google.com/g/2005#kind" ]

        # retrieve comments, if required
        if max_comment_count > 0:
            yt_dict['comments'] = self.get_video_comments(yt_dict['id'],
                                                          max_comment_count)

        return yt_dict

    @classmethod
    def _get_yt_dict(cls, entry, mapping):
        ''' stores the mapped items in a dictionary 
        @param entry: Gdata Entry
        @param mapping: dictionary with the mapping
        @return: yt_dict
        '''
        yt_dict = {}
        for attr, key in mapping.iteritems(): 
            try: 
                yt_dict[key] = attrgetter(attr)(entry)
                
                if key in ('published', 'last_modified'):
                    yt_dict[key] = convert_date(yt_dict[key])
                
            except AttributeError, e:
                logger.warn('AttributeError: %s' % e)
                yt_dict[key] = None        
        
        return yt_dict

    def get_video_comments(self, video_id, max_comments=25):
        """ @param video_id: the video_id of the video for which we want to
                             retrieve the comments.
            @param max_comments: maximum number of comments to retrieve
            @return: a list of comments 
        """
        comments = []
        comment_feed = self.youtube_service.GetYouTubeVideoCommentFeed(
                            video_id=video_id)
        
        while comment_feed:
            
            sleep( YOUTUBE_SLEEP_TIME )
            for comment in comment_feed.entry:
                url, in_reply_to = self.get_relevant_links(comment)
                yt_dict = self._get_yt_dict(comment, self.YT_COMMENTS_MAPPING)
                yt_dict.update({'url': url,
                                'in-reply-to': in_reply_to,
                                'author': comment.author[0].name.text,})
                comments.append(yt_dict)

                if len(comments) == max_comments:
                    return comments
                
            # retrieve the next comment page, if available
            if not comment_feed.GetNextLink():
                break
           
            comment_feed = self.youtube_service.Query(
                                    comment_feed.GetNextLink().href)
        
        return comments


    @staticmethod
    def get_relevant_links(comment):
        """ 
        @param comment: a single YouTube comment.
        @return: a tuple indicating:
                 a) the comment's url, and 
                 b) the url of the comment to which this comment refers (in case
                    it is a reply)
        """
        comment_url, in_reply_to = None, None
        for link in comment.link:
            if link.rel == 'self':
                comment_url = link.href
            elif link.rel.endswith("#in-reply-to"):
                in_reply_to = link.href

        return comment_url, in_reply_to        

                                    
class YouTubeTest(unittest.TestCase):
        
#     def setUp(self):
#         self.search_terms = ["Linus Torvalds","Ubuntu"]
#         self.youtube = YouTube()
#         logger.addHandler(logging.StreamHandler())
    
    
    def test_search_v3(self):
        search_term = 'FIFA'
        api_key = 'AIzaSyAfFOJnJMd3QfP0cV9HDymin19gkYDDR5c'
        
        service = YouTube_v3(api_key)
        result = service.search(search_terms=search_term)
        for item in result:
            print item
        print result
        
#     def test_query_time(self):
#         test_cases = ((1000, 'today'), 
#                       (5000, 'this_week'), 
#                       (12000, 'this_month'), 
#                       (datetime.now() - timedelta(hours=12), 'today'),
#                       (datetime.now() - timedelta(days=4), 'this_week'),
#                       (datetime.now() - timedelta(days=15), 'this_month'),)
#         
#         for max_age, exp_result in test_cases: 
#             result = YouTube.get_query_time(max_age)
#             assert exp_result == result, 'max_age %s, result %s, exp %s' % (max_age, 
#                                                                             result, 
#                                                                             exp_result)
#     def test_search(self):   
#         required_keys = ('location', 'content', 'id', 'url', 'title',
#                          'last_modified', 'published', 'user_name', 
#                          'yt_source', 'rights', 'summary', 'keywords', 
#                          'related_url', 'statistics_viewcount', 
#                          'statistics_favoritecount', 'rating_average', 
#                          'rating_max', 'rating_min', 'rating_numraters', 
#                          'picture', 'duration', 'user_url'
#                         )
#              
#         for r in self.youtube.search(self.search_terms, None, orderby='relevance'):
# 
#             assert len(required_keys) == len(r.keys())
#             assert isinstance(r['last_modified'], datetime)
#             for rk in required_keys: 
#                 if not rk in r.keys():
#                     print 'k ', sorted(r.keys())
#                     print 'rk', sorted(required_keys)
#                     assert False, 'key %s missing' % rk
#     
#     def test_convert_date(self):
#         date_str = '2012-11-24T02:48:24.000Z'
#         assert convert_date(date_str) == datetime(2012, 11, 24, 2, 48, 24)
# 
#     def test_comments_result(self):
#         search_terms = ((('Linux',), 5), (('Climate Change',), 3), 
#                         (('Microsoft',), 2))
#         
#         for search_term, max_results in search_terms:
#             print 'querying youtube for %s' % search_term
#             result = self.youtube.search(search_term, None, max_results, 
#                                          orderby='relevance', 
#                                          max_comment_count=5)
#             print '\t got %s documents, max_results was %s' % (len(result),
#                                                                max_results) 
#             self.assertEqual( len(result), max_results)
#             
#             num_comments = max([ len(r['comments']) for r in result ])
#             print "Maximum number of comments for search term '%s': %d" % (search_term[0], num_comments)
#             self.assertEqual( 5, num_comments )
#             print '-------------------------'
#             
# 
#     def test_paging(self):
#         """
#         verifies that the results contain the necessary fields.
#         <ul>
#           <li>ratings: if not disabled by the publisher, _most_ videos
#                        should contain ratings </li>
#         </ul>
#         """
#         MAX_RESULTS = 70
#         result = self.youtube.search( ("Linux", ), None, 
#                                 max_results=MAX_RESULTS, orderby='relevance')
# 
#         # verify that all videos have been retrieved
#         assert len(result) == MAX_RESULTS
# 
#         # verify that the video rating and keywords have been correctly retrieved
#         # and interpreted
#         ratings_retrieved = 0
#         for _, r in enumerate(result):
#             if r['rating_average']:
#                 ratings_retrieved += 1
#             if r['keywords']:
#                 assert isinstance(r['keywords'][0], str)
#                 
#         self.assertGreater(ratings_retrieved, MAX_RESULTS*0.9, 
#                    "Assert that at least 90% of all videos contain ratings")
# 
# 
#     def test_search_term_list(self):
#         self.assertRaises( ValueError, self.youtube.search, "Linux" )
#        
# 
#     def test_comments(self):
#         comments = self.youtube.get_video_comments(video_id="yI4g8Ti6eTM", 
#                                                    max_comments = 27)
#         assert len(comments) == 27



        

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python

# (C)opyrights 2008-2009 by Albert Weichselbraun <albert@weichselbraun.net>
#                           Heinz-Peter Lang <heinz@langatium.net>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

__version__ = "$Header$"

import re, logging, sys
from eWRT.access.http import Retrieve
from eWRT.ws.TagInfoService import TagInfoService
import time
from lxml import etree
from datetime import timedelta, date

SLEEP_TIME = 5
NON_ABSTRACT_TEXT = ['a', 'h3']
RESULTS_PER_PAGE = 10

cleanText = lambda text: trim(re.sub('\n', ' ', text))

logger = logging.getLogger('Technorati')
handler = logging.StreamHandler()
logger.addHandler(handler)

class Technorati(TagInfoService):
    """ retrieves data using the del.icio.us API """

    RE_TAG_COUNT = re.compile('Posts relating to &ldquo;.*?&rdquo; \((\S+)\)',
        re.IGNORECASE | re.DOTALL)
    RE_TAG_CONTAINER = re.compile('<div id="related-tags".*?<ul>(.*?)</ul>.*?</div>',
        re.IGNORECASE | re.DOTALL)
    RE_RELATED_TAG = re.compile('<a.*?">(.*?)</a>',
        re.IGNORECASE | re.DOTALL)

    AVAILABLE_TOPICS = ['overall', 'entertainment', 'business', 'sports', 'politics', 'autos', 'technology', 'living', 'green', 'science']
    AVAILABLE_AUTHORITY = ['all', 'high', 'medium', 'low']
    AVAILABLE_SOURCE = ['advanced-source-blogs', 'advanced-source-news', 'advanced-source-all']
    AVAILABLE_RETURN = ['posts', 'news']

    TOPIC = 'overall'
    AUTHORITY = 'high'
    SOURCE = 'advanced-source-all'
    RETURN = 'posts'

    TECHNORATI_URL = 'http://technorati.com/search?usingAdvanced=1&q=%s&return=%s&source=%s&topic=%s&authority=%s'

    __slots__ = ()
    last_access = 0


    @staticmethod
    def getTagInfo(tags):
        """ @param   tags   A list of tags to retrieve information for
            @returns        the number of bookmarks using the given tags
        """
        url = Technorati._parseURL(tags)
        content = Technorati.get_content(url)

        return Technorati._parse_tag_counts(content)


    @staticmethod
    def getRelatedTags(tags, withCounts=True):
        """ @param tags - list of tags
            @param booelan withCounts to print counts for tags
            @return list of related tags 
        """

        # not supported at the moment

        return []

        url = Technorati._parseURL(tags)

        content = Technorati.get_content(url)
        tag_container = Technorati.RE_TAG_CONTAINER.findall(content)
        related_tags_with_count = []

        if len(tag_container) > 0:

            related_tags = Technorati.RE_RELATED_TAG.findall(tag_container[0])
            if withCounts:
                for tag in related_tags:
                    tag = re.sub(' ', '-', tag)
                    related_tags_with_count.append((tag, Technorati._getRelatedTagCount(tags, tag)))

                return related_tags_with_count

            else:
                return related_tags



    # 
    # helper functions
    #
    @staticmethod
    def _parse_tag_counts(content):
        """ parses technorati html content and returns the number of counts for the tags 
            @param content: HTML content of technorati
            @return: count of related posts
        """

        m = Technorati.RE_TAG_COUNT.search(content)
        if m:
            return re.sub(',', '', m.group(1))
        else:
            return 0

    @staticmethod
    def _getRelatedTagCount(tags, filter):
        """ returns the related tag count
            @param tags - list of tags
            @param filtered tag
            @return count of related tag """

        url = Technorati._parseURL(tags, filter)
        content = Technorati.get_content(url)
        return Technorati._parse_tag_counts(content)

    @staticmethod
    def _parseURL(tags, filter=''):
        """ parses the URL """

        if type(tags).__name__ == 'str':
            url = Technorati.TECHNORATI_URL % (tags, Technorati.RETURN, Technorati.SOURCE, Technorati.TOPIC, Technorati.AUTHORITY)
        else:
            url = Technorati.TECHNORATI_URL % ("+".join(tags), Technorati.RETURN, Technorati.SOURCE, Technorati.TOPIC, Technorati.AUTHORITY)

        if filter == '':
            return url
        else:
            return '%s&filter=%s' % (url, filter)


    @staticmethod
    def get_content(url):
        """ returns the content from Technorati """
        assert(url.startswith("http"))

        logger.debug('Fetching content for URL %s' % url)

        if (time.time() - Technorati.last_access) < SLEEP_TIME:
            logger.debug('Sleeping %s seconds!' % SLEEP_TIME)
            time.sleep(SLEEP_TIME)

        Technorati.last_access = time.time()

        f = Retrieve("%s_new" % Technorati.__name__).open(url)
        content = f.read()
        f.close()
        return content

    @staticmethod
    def get_blog_links (searchTerm, maxResults=100, offset=0, maxAge=0, 
                        country=None):
        '''
        Searches Technorati for the given term and returns a list of links 
        to the blogs
        @param searchTerm: term to search
        @param maxResults: max search results 
        @param offset:
        @param maxAge: maximum age of the articles in days,
            if set links are ordered by relevance and not by date
        @param country: currently not supported
        @return: dictionary with all the relevant info
        '''

        if isinstance(searchTerm, list):
            searchTerm = '+'.join(searchTerm)

        searchTerm = re.sub(' ', '+', searchTerm)
        if offset >= RESULTS_PER_PAGE:
            page = (offset / RESULTS_PER_PAGE) + 1
        else:
            page = 1

        if maxAge == 0: orderBy = 'relevance'
        else: orderBy = 'date'

        url = '%s&page=%s&sort=%s' % (Technorati._parseURL(searchTerm), 
                                      page, orderBy)
        content = Technorati.get_content(url)
        print '.'
        return Technorati._parseLinksFromContent(content, searchTerm, 
                                                 maxResults=maxResults, 
                                                 offset=offset, maxAge=maxAge)


    @staticmethod
    def _parseLinksFromContent(content, searchTerm, maxResults=100, 
                               offset=0, maxAge=0):
        ''' parses the links from the content '''
        tree = etree.HTML(content)
        links = []
        resultList = tree.xpath('//ol[@class="search-results post-list"]')

        logger.debug('Parsing links from Technorati result: %s, %s, %s, %s ' % (searchTerm, maxResults, offset, maxAge))

        if len(resultList) > 0:
            counter = 0
            for element in resultList[0].iterchildren():
                blogLink = {}
                linkDate = Technorati._getDate(element, maxAge)
                if not linkDate == None:

                    try:
                        blogLink['url'] = element.xpath('.//a[@class="offsite"]')[0].attrib['href']
                        logger.info('Found url %s' % blogLink['url'])
                    except IndexError:
                        logger.error('Could not parse the URL')

                    try:
                        blogLink['authority'] = int(re.sub('Authority ', '', element.xpath('.//a[@class="authority"]')[0].text))
                    except:
                        logger.error('Could not parse the authority!')
                        blogLink['authority'] = ''

                    blogLink['source'] = 'Technorati - Keyword "%s"' % searchTerm
                    blogLink['abstract'] = Technorati._getAbstract(element)
                    blogLink['reach'] = '0'
                    blogLink['date'] = linkDate

                    if blogLink.has_key('url'):
                        logger.debug('Found URL %s' % blogLink['url'])
                        links.append(blogLink)
                        offset += 1
                        counter += 1
                        if offset == maxResults:
                            break
                    else:
                        offset += 1

            if offset < maxResults and counter == RESULTS_PER_PAGE:
                links.extend(Technorati.get_blog_links(searchTerm, maxResults, 
                                                       offset, maxAge))

        else:
            if offset == 0:
                logger.warning('At the first page - should find a result for %s -> Changes in the site structure' % searchTerm)

        return links

    @staticmethod
    def _getAbstract(element):
        ''' gets the abstract from the element '''
        text = []

        if not element.tag in NON_ABSTRACT_TEXT and not element.text == None:
            text.append(element.text.lstrip())

        for child in element.getchildren():
            text.extend(Technorati._getAbstract(child))

        if not element.tag in NON_ABSTRACT_TEXT and not element.tail == None:
            text.append(element.tail.lstrip())

        return ''.join(text)

    @staticmethod
    def _getDate(element, maxAge=0):
        ''' returns the date of the link '''

        p = re.compile('(\d+)\s(\w+)\sago')

        minDate = (date.fromtimestamp(time.time()) - timedelta(days=maxAge))
        linkDate = None
        for div in element.xpath('.//div'):
            m = p.search(div.text)
            if m:
                age = int(m.groups()[0])
                timeDim = m.groups()[1]

                if re.search('week.?', timeDim):
                    hours = age * 7 * 24
                elif re.search('day.?', timeDim):
                    hours = age * 24
                elif re.search('hour.?', timeDim):
                    hours = age
                elif re.search('minute.?', timeDim):
                    hours = 1
                else:
                    hours = 0

                if hours > 0:
                    linkDate = date.fromtimestamp(time.time()) - timedelta(hours=hours)

                    if linkDate < minDate and maxAge > 0:
                        linkDate = None
            else:
                logger.warning("Could not find a date")

        return linkDate


if __name__ == '__main__':

    print Technorati.getRelatedTags(("Linux",))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import simplejson

from eWRT.ws.WebDataSource import WebDataSource

import oauth2
from atom import Link

class LinkedIn(WebDataSource):
    
    def __init__(self, consumer_key, consumer_secret, user_token, user_secret):
        WebDataSource.__init__(self)
        
        consumer = oauth2.Consumer(consumer_key, consumer_secret)
        access_token = oauth2.Token(key=user_token, secret=user_secret)
        self.client = oauth2.Client(consumer, access_token) 
        self.linkedin_uri = """http://api.linkedin.com/v1/"""
        
    def search(self, search_terms, companies=False, jobs=False, option='keywords', 
                    format='json'):
        
        '''Search linkedin for companies or jobs.
        The number of search results is limited to 10 items per search term.
        
        Usage::
        >>> linkedin_obj = LinkedIn(consumer_key, consumer_secret, user_token, user_secret)
        >>> linkedin_obj.search(['IBM', 'Microsoft'], companies=True)
        >>> linkedin_obj.search(['Python', 'Machine Learning'], jobs=True)
        
        If you need a more specific search you can use certain options instead
        of the option keywords or you place a custom search request::
        
        >>> linkedin_obj.search(['Python', 'Machine Learning'], jobs=True, option='job-title')
        >>> linkedin_obj.custom_search('http://api.linkedin.com/v1/company-search:(facets)?keywords={Python}&facets=location')
        '''
        assert isinstance(search_terms, list)
        assert not (companies and jobs) and (companies or jobs)
        
        retval = None
        
        if companies:
            retval = self.search_companies(search_terms, format)
        elif jobs:
            retval = self.search_job_by_search_terms(search_terms, format)
        
        assert retval
        return retval

    def custom_search(self, uri, format='json'):
        self._assert_basestring(uri)
        self._assert_format(format)
        
        response, content = self.client.request(uri, 
                                                headers={'x-li-format':format})
        
        return simplejson.loads(content)

    def search_company(self, search_term, option='keywords', format='json'):
        '''Search for companies using one search term'''
        self._assert_basestring(search_term, option)
        self._assert_format(format)
        
        request = self.linkedin_uri + 'company-search?%s=%s' % (option,
                                                                search_term)
        response, content = self.client.request(request, 
                                                headers={'x-li-format':format})
            
        return self._convert_json_to_dict(content, 'companies')
    
    
    def search_companies(self, search_terms, option='keywords', format='json'):
        '''Search for companies using several keywords'''
        
        assert isinstance(search_terms, list)
        self._assert_basestring(option)
        self._assert_format(format)
        
        results = []
        for search_term in search_terms:
            results.append(self.search_company(search_term, option, format))
            
        return results
    
    
    def search_job_by_search_term(self, search_term, option='keyword', format='json'):
        '''Search for a job using a certain search term'''
        
        self._assert_basestring(search_term, option)
        self._assert_format(format)
        assert isinstance(search_term, basestring)
        
        request = self.linkedin_uri + 'job-search?%s=%s' % (option, search_term)
        response, content = self.client.request(request, 
                                                headers={'x-li-format':format})
        
        return self._convert_json_to_dict(content, search_field='jobs')
    
    def search_job_by_search_terms(self, search_terms, option='keywords', 
                                            format='json'):
        '''Search for jobs using several search terms.'''
        self._assert_format(format)
        assert isinstance(search_terms, list)
        self._assert_basestring(option)
        
        results = []
        for search_term in search_terms:
            results.append(self.search_job_by_search_term(search_term, option, 
                                                          format))
        
        return results

    def _convert_json_to_dict(self, json_string, search_field):
        self._assert_basestring(json_string, search_field)
        
        raw_data = simplejson.loads(json_string)

        results = []        
        for key, value in raw_data.items():
            
            if key == search_field:
                for item in value['values']:
                    results.append(item)
        
        return results


    def _assert_format(self, format):
        assert format == 'json' or format == 'xml'
        
    def _assert_basestring(self, *args):
        for possible_string in args:
            assert isinstance(possible_string, basestring)
            assert len(possible_string) > 0
                    



'''
Created on 21.06.2012

@author: Norman Suesstrunk, Heinz-Peter Lang

class for executing batch requests to the facebook api
'''
import unittest
import json
import urllib
import httplib
import logging
from ssl import SSLError

from eWRT.config import FACEBOOK_ACCESS_KEY
from eWRT.ws.facebook import FacebookWS

MAX_BATCH_SIZE = 3
TIMEOUT = 100 # timeout for the requests in seconds

logger = logging.getLogger('eWRT.ws.facebook')

class FbBatchRequest(object):
    '''
    @class FacebookBatchRequest
    Class for sending batch requests to the facebook api
    the actual requests are constructed with objects from the FacebookWS class.  
    '''

    batchHTTPParam = 'batch' # http post parameter for the facebook batch http interface
    accessTokenHTTPParam = 'access_token'
    faceBookGraphHost = 'graph.facebook.com'

    def __init__(self, access_token=FACEBOOK_ACCESS_KEY):
        ''' Constructor '''
        self.access_token = access_token
    
    def run_search(self, terms, objectType='all', since=None, limit=None):
        ''' runs a batch search '''
        if not isinstance(terms, list):
            terms = [terms]

        return self.make_search([FacebookWS(term, objectType, 
                                            since, limit) for term in terms])

    def make_search(self, fbWSList):
        ''' executes the search 
        :param fbWSList: list of :class:eWRT.ws.facebook.FacebookWS object
        :type fbWSList: list
        :returns: result
        :rtype: list
        ''' 
        assert len(fbWSList), 'list of facebook services empty'

        result = []
        search_result = self._send_post(self.access_token, fbWSList)
        
        for row in search_result: 
            if not row: 
                logger.debug('row == %s ... continue' % row)
                continue
            
            data = json.loads(row['body'])
            
            if 'data' in data:    
                for post in json.loads(row['body'])['data']:
                    post['url'] = 'http://www.facebook.com/%s' % post['id']
                    result.append(post)
                    if 'comments' in post and 'data' in post['comments']:
                        for i, comment in enumerate(post['comments']['data']):
                            comment['type'] = 'comment'
                            comment['parent_url'] = post['url']
                            comment['url'] = '%s?comment=%s' % (post['url'], i)
                            result.append(comment)
            elif data: 
                result.append(data)
                
        return result

    @classmethod
    def _get_json_batch_request_string(cls, fbWSList):
        '''
        delivers the json-string in the apropriate format for the facebook batch api
        '''
        return [fb.getJsonListStructure() for fb in fbWSList]
    
    @classmethod
    def _send_post(cls, access_token, fbWSList):
        '''
        sends the batch request as post to the facebook batch request api
        returns all the search results for the batch request
        '''
        result = []
        conn = httplib.HTTPSConnection(cls.faceBookGraphHost,
                                       timeout=TIMEOUT)
        all_batch_requests = cls._get_json_batch_request_string(fbWSList)
        
        for batch_requests in cls.get_batch(all_batch_requests):
            batch_requests = json.dumps(batch_requests)
            logger.debug('Making batch_requests with %s' % batch_requests)
            params = urllib.urlencode({cls.accessTokenHTTPParam: access_token,
                                       cls.batchHTTPParam: batch_requests})
            headers = {'Content-type': 'application/x-www-form-urlencoded',
                       'Accept': 'text/plain'}
            
            try: 
                
                conn.request('POST', '/', params, headers)
                response = conn.getresponse()
    
                data = json.loads(response.read())
    
                if isinstance(data, dict) and 'error' in data:
                    result.append(data)
                else: 
                    result.extend(data)

            except SSLError, e: 
                logger.error('Could not request: %s, %s' % (batch_requests, e))
            
        conn.close()
        return result
    
    @staticmethod
    def get_batch(requests, batch_size=MAX_BATCH_SIZE):
        for i in range(0, len(requests), batch_size):
            yield requests[i:i+batch_size]
            
class TestFacebookWS(unittest.TestCase):
    
    def test_bad_request(self):
        access_token = "AAAElj9ZBZCquoBAGkKlcPvJsUCpyAZBxz6nsOYr8LAmpIj9Q9EZCKl9xVAYmlXGh2UQvhVellSWsZALPn6V73ZAZBiaxlwqkWlUjGVLzAHd7gZDZD"
        fbBatchRequest = FbBatchRequest(access_token)
        
        try: 
            fbBatchRequest.run_search('Linus Torvalds')
        except Exception, e: 
            print 'thats ok: %s' % e
            assert True
    
    def test_batch_search2(self):
        fbBatchRequest = FbBatchRequest()
        result = fbBatchRequest.run_search(['Wien'], 'post', 100)
        assert len(result) > 0
    
    def test_feed_mirroring(self):
        fbBatchRequest = FbBatchRequest()
        result = fbBatchRequest.run_search('107961012601035/feed', 
                                           objectType='path',
                                           limit=10)
        
        for x in result:
            print x
            
        assert len(result) >= 1

    def test_search(self):
        terms = ['Department of Health and Human Services', 'carcinomas', 
                 'National Oceanic and Atmospheric Administration', 'Mercedes-Benz',
                 'Audi', 'BMW', 'tumour', 'cancer', 'cop18', 'jane lubchenco', 'tumour', 'irgendwas']
        
        fbWSList = []
        
        for term in terms: 
            fbWSList.append(FacebookWS(term, 'post', 1353954200, 100))
        
        result = FbBatchRequest._send_post(fbWSList)
        print result
        print len(result)
        
if __name__ == "__main__":

    unittest.main()



#!/usr/bin/env python

""" @package eWRT.util.timing
    timing of abitrary method calls

    Examples: see unittests
"""

# (C)opyrights 2010 by Albert Weichselbraun <albert@weichselbraun.net>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

__author__    = "Albert Weichselbraun"
__revision__  = "$Id$"
__copyright__ = "GPL"

from time import time
from unittest import TestCase

class Timed(object):
    """ decorator class used to time functions """

    def __init__(self, f):
       self.f = f
       self.clear()

    def __call__(self, *args, **kargs):
        """ calls the timed function and computes all necessary
            statistics """
        self.numberOfCalls += 1
        beforeCall= time()
        retVal = self.f(*args, **kargs)
        self.lastCallDuration   = time()-beforeCall
        self.totalCallDuration += self.lastCallDuration
        return retVal

    def clear(self):
        self.startTime         = time()
        self.lastCallDuration  = None
        self.totalCallDuration = 0.
        self.numberOfCalls     = 0
 

class TimedTest(TestCase):

    @Timed
    def _timeFunction(self, a, b):
        [ x for x in xrange(1000) ]
        return a + b

    def testReturnValue(self):
        print self._timeFunction(self, 2,8)
        assert self._timeFunction(self, 2, 8) == 10

    def testCallStatistics(self):
        NR_CALL = 5000
        for x in xrange(NR_CALL):
            self._timeFunction(self, x, 10*x)
        
        assert self._timeFunction.numberOfCalls == NR_CALL
        self.assertAlmostEqual( self._timeFunction.totalCallDuration/NR_CALL, self._timeFunction.lastCallDuration, 1 )
        
    def testClearCallStatistics(self):
        NR_CALL = 5000
        for x in xrange(NR_CALL):
            self._timeFunction(self, x, 10*x)

        self._timeFunction.clear()
        assert self._timeFunction.totalCallDuration == 0.
        assert self._timeFunction.numberOfCalls == 0
        



        
# $Id$



from __init__ import Calais
import unittest



class TestCalais( unittest.TestCase ):
    ''' Testing GoogleTrends webservice '''
    
    test_html = '''  
  <html>
  <head>
  <title>test</title>
  </head>
  <body>
  <div class="content">
    <h1>Input Content</h1>
<h2>Language</h2>
<p>OpenCalais today supports content in English, French and Spanish. OpenCalais applies a Language Identification module before processing the text for entities, events and facts. This module might fail to recognize the language if the submitted text length is small.</p>
<p>If the submitted content is less than 100 characters and the language cannot be recognized, OpenCalais will assume the language is English by default and and the English module will process the text for entities, events and facts. In addition, in such cases, OpenCalais will return &quot;Input Text Too Short&quot; as the language code in the RDF.</p>
<h2>HTML Cleaning</h2>
<p>A document coded in HTML usually contains much more information than what is perceived as &ldquo;new and relevant textual content&rdquo;. This information might be:</p>

<ul type="disc">
<li>The HTML tags</li>
<li>Various metadata</li>
<li>Browsing menus</li>
<li>Advertisements</li>
<li>Links to other service providers like Facebook or Digg.</li>
<li>Links to other HTML pages on the same site that are not part of the main content frame.</li>
<li>General information about the hosting site, including copyright notice, etc.</li>
</ul>
<p>In order to optimize extraction and categorization results or, for that matter, any semantic-related algorithm operating on the text, we opt for the text to be as free as possible from the above objects.</p>

<p>The HTML cleaner aims to clean the text as much as possible of these objects, leaving only a basic HTML structure that contains only the new and relevant data. It does so using both HTML-structural and machine-learning-based heuristics.&nbsp;</p>
<h2>Format&nbsp;</h2>
<p>OpenCalais supports four formats of content: TEXT/HTML, TEXT/HTMLRAW, TEXT/XML and TEXT/RAW.</p>
<p>When no content type is specified, Calais attempts to auto-detect the type (one of: TEXT/XML, TEXT/HTML or TEXT/RAW).</p>
<p><strong><em>Note:</em></strong><em> </em>TEXT/TXT is no longer used. If this type is given, it is processed as though the contentType value was TEXT/RAW and the results appears as TEXT/RAW.</p>
<p><strong>TEXT/HTML:</strong> No conversion of the submitted content, will remove irrelevant text content as well as HTML tags and scripts. Entity and event detection will be relative to the cleansed text. For optimal results it is recommended to use this contentType when submitting HTML content.</p>
<p><strong>TEXT/HTMLRAW:</strong> Will apply OpenCalais' legacy converter and limited cleansing, removing only HTML tags and scripts, This is equivalent to TEXT/HTML of previous versions.</p>

<p><strong>TEXT/XML:</strong> Will apply the XML converter for escaping the necessary characters, hence entity and event detection will be relative to the cleansed text. For optimal results it is recommended to use this contentType when submitting XML content. The XML converter also supports the NewsML standard.</p>
<p>If the content is submitted as TEXT/XML, OpenCalais will process the following XML nodes:</p>
</div>
</body>
</html>
 ''' 
    
    def setUp(self):
        self.c = Calais('heinz')
        

    def testAnalyze(self):
        things = self.c.analyze(self.test_html, 'text/html')
        print things
        assert len(things) == 6

if __name__ == '__main__':
    unittest.main()


from __init__ import Flickr
import unittest

class TestFlickr( unittest.TestCase ):
    ''' Testing Flickr webservice '''
    
    def testMultipleTags( self ):
        """ verifies whether multiple tags return results different from single tags """
        
        print '### Testing multiple ###'
        
        assert Flickr.getRelatedTags( ("berlin", "dom")) != Flickr.getRelatedTags( ("berlin",) ) 
        assert Flickr.getTagInfo( ("berlin", "dom") ) != Flickr.getTagInfo( ("berlin",) ) 

    def testTagInfo( self ):
        """ test the tag info """
        
        print '### Testing tag info ###'
        
        for tags in ( ('berlin','dom') , ('vienna',),  ('castle',)):
            count = Flickr.getTagInfo( tags )
            print '%s has tag count: %s' % (tags, count )
            assert count > 0

    def testRelatedTags( self ):
        """ test related tags by retrieving related tags for the following tags """
        
        print '### Testing related tags ###'
        
        tags = ('berlin', 'dom')
        count = Flickr.getRelatedTags( tags )
        print '%s has related tags: %s' % (tags, Flickr.getRelatedTags( tags ))
        assert count > 0
        count = Flickr.getRelatedTags( tags )
        print '%s has related tags: %s' % ('berlin', Flickr.getRelatedTags( tags ))
        assert count > 0

if __name__ == '__main__':
    unittest.main()


def info(object, spacing=10, collapse=1):
    """Print methods and doc strings.
    Takes module, class, list, dictionary, or string."""
    methodList = [method for method in dir(object) if callable(getattr(object, method))]
    processFunc = collapse and (lambda s: " ".join(s.split())) or (lambda s: s)
    print "\n".join(["%s %s" %
                      (method.ljust(spacing),
                       processFunc(str(getattr(object, method).__doc__)))
                     for method in methodList])
if __name__ == "__main__":               
    print info.__doc__ 


from __init__ import GoogleTrends, LOADED

class TestGoogleTrends( object ):
    ''' Testing GoogleTrends webservice '''
    
    def setUp(self):
        if LOADED:
            self.trends = GoogleTrends()
        
    def testGetTrend(self):
        ''' test getting trends'''
        if LOADED:
            trends = self.trends.getTrends(('Terminator', 'Rambo'))
            print trends
            assert len(trends) > 52
            
            trends =  self.trends.getTrends(('Microsoft', 'Linux', 'Apple'))
            print trends
            assert len(trends) > 52





#!/usr/bin/env python

""" unittests covering the del.icio.us functions provided by eWRT """

# (C)opyrights 2008 by Albert Weichselbraun <albert@weichselbraun.net>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


from __init__ import Delicious
import unittest

DELICIOUS_TEST_URLS = ( 'http://www.iaeste.at', 'http://www.wu-wien.ac.at', 'http://www.heise.de', 
                        'http://www.kurier.at', 'http://news.bbc.co.uk', )

DELICIOUS_TEST_TAGS = [("linux",), ("information",), ("information", "retrieval"),("linux", "debian"),("algore",)]

class TestDelicious( unittest.TestCase ):

    def test_url_info(self):
        for url in DELICIOUS_TEST_URLS:
            print '%s has %s counts '% (url, Delicious.delicious_info_retrieve(url))

    def test_tag_info(self):
        print '### Testing tag_info ###'
        for tags in DELICIOUS_TEST_TAGS:
            print '%s has %s counts ' % (tags, Delicious.getTagInfo( tags ))

    
    def test_related_tags(self):
        print '### Testing related_tags ###'
        for tags in DELICIOUS_TEST_TAGS:
            print '%s has related tags: %s' % (tags, Delicious.getRelatedTags( tags ))

if __name__ == '__main__':
    unittest.main()




#!/usr/bin/env python
"""
 @package eWRT.ws.stat.coherence
 Determines how strongly two terms are connected to each other
"""

# (C)opyrights 2010 by Albert Weichselbraun <albert@weichselbraun.net>
#                      Johannes Duong <johannes.duong@wu.ac.at>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from eWRT.ws.TagInfoService import TagInfoService
from unittest import TestCase
from eWRT.util.cache import DiskCache

from math import exp

class Coherence(object):
    """ @class Coherence
        abstract class for computing the coherence between 
        terms """

    def __init__(self, dataSource, cache=True):
        """ @param[in] dataSource implementing the TagInfoService Interface """
        assert isinstance(dataSource, TagInfoService) 
        self.dataSource  = dataSource
        if cache==True:
            diskCache       = DiskCache("./.coherence-tagcount-cache", 2)
            self.getTagCount = lambda tt: diskCache.fetchObjectId( self.dataSource.__class__.__name__ + str(tt), 
                                                                   self.dataSource.getTagInfo, tt)
        else:
            self.getTagCount = self.dataSource.getTagInfo

    @staticmethod
    def getCoherence(nx, ny, nt):
        """ @param[in] nx  counts of term1
            @param[in] ny  counts of term2
            @param[in] nt counts of term1 together with term2
            @returns the coherence
        """
        raise NotImplemented

    def getTermCoherence(self, t1, t2):
        """ @param[in] t1 term1
            @param[in] t2 term2
            @returns the coherence between these two terms
        """        
        nx = self.getTagCount( (t1, ) )
        ny = self.getTagCount( (t2, ) )
        nt = self.getTagCount( (t1, t2) )
        return self.getCoherence(nx, ny, nt)


class DiceCoherence(Coherence):
    """ @class DiceCoherence
        computes the dice coherence for the given terms """

    @staticmethod
    def getCoherence(nx, ny, nt):
        """ @param[in] nx  counts of term1
            @param[in] ny  counts of term2
            @param[in] nt counts of term1 together with term2
            @returns the coherence
        """
        try:
            return 2*float(nt)/float(ny+ny)
        except ZeroDivisionError:
            return None

class PMICoherence(Coherence):
    """ @class PMICoherence
        computes the coherence based on the pointwise mutual
        information (PMI)
    """

    @staticmethod
    def getCoherence(nx, ny, nt):
        """ @param[in] nx  counts of term1
            @param[in] ny  counts of term2
            @param[in] nt counts of term1 together with term2
            @returns the coherence
        """
        nx, ny, nt = float(nx), float(ny), float(nt)

        nz = nx + ny + nt
        try:
            fx = (nx/nz)*exp((nx/nz)*-1)
            fy = (ny/nz)*exp((ny/nz)*-1)
            ft = (nt/nz)*exp((nt/nz)*-1)
            return ft/(fx*fy)
        except ZeroDivisionError:
            return None
        


class TestCoherence(TestCase):
    
    def testDice(self):
        """ tests the computation of the dice coefficient 
            based on the example in
              http://en.wikipedia.org/wiki/Dice's_coefficient
        """
        assert DiceCoherence.getCoherence( 4, 4, 1 ) == 0.25

    def testPMI(self):
        """ tests the computation of the PMI based on the results
            from wilson's paper 
        """
        c = PMICoherence.getCoherence
        self.assertAlmostEqual( c(4710000000, 125000, 897), 0.0195069043716 ) # one - coup whole wheat flour
        self.assertAlmostEqual( c(3670000, 870000, 897), 0.00346634415814)    # one coup - whole wheat flour
        self.assertAlmostEqual( c(4270, 2690000, 897), 0.571746307316)        # one coup whole - wheat flour
        self.assertAlmostEqual( c(2320, 33400000.0, 897), 1.05103564089)      # one coup whole wheat - flour
        
    def testPMIZero(self):
        """ tests the handling of PMI values of no counts are found """
        c = PMICoherence.getCoherence
        assert c(0,12,0) == None




#!/usr/bin/env python

""" @package eWRT.config
    evaluates ~/.eWRT/siteconfig.py and publishes the values"""

# (C)opyrights 2004-2013 by Albert Weichselbraun <albert@weichselbraun.net>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
from warnings import warn
from os.path import expanduser, exists, dirname, join as os_join

SYS_EWRT_CONF = "/etc/eWRT/sysconfig.py"
USR_EWRT_CONF = expanduser("~/.eWRT/siteconfig.py")

# commands used to convert various formats to text
CMD_CONV = {'html': '/usr/bin/lynx -stdin -width=20000 -force_html -nocolor -dump -nolist -nobold -pseudo_inlines=0 -assume_charset=%s -display_charset=utf8',
            'pdf' : '/usr/bin/pdftotext -layout - - ',
            'doc' : '/usr/bin/antiword -',
            }

# ===================================================================
# DEFAULT CONFIGURATION VALUES
# - please either create a configuration file in 
#     /etc/eWRT/sysconfig or
#     ~/.eWRT/siteconfig.py
#
#   to overwrite these settings
# ===================================================================


PROXY_SERVER = ''
USER_AGENT = 'eWRT Version/0.5; MODUL <%s> +http://p.semanticlab.net/eWRT'

# default sleep time in seconds
DEFAULT_WEB_REQUEST_SLEEP_TIME = 1

# ===================================================================
# DATABASE CONFIGURATION
# ===================================================================
DATABASE_CONNECTION = {
            'db-name'  : {'host': 'localhost', 'dbname': 'postgres', 'username': '', 'passwd': ''},
    }
 
# ===================================================================
# USERNAMES AND API-KEYS
# ===================================================================

# delicious
DELICIOUS_USER = ''
DELICIOUS_PASS = ''

# opencalais
OPENCALAIS_KEY = ''
OPENCALAIS_CACHE_DIR = ''
OPENCALAIS_URL = "http://api.opencalais.com/enlighten/calais.asmx/Enlighten"

# geoTagger
GEOLYZARD_URL = ''
GEOLYZARD_GAZETTEERS = ''


# amazon
AMAZON_ACCESS_KEY_DICT = { 'user': 'pass',
                         }
AMAZON_ACCESS_KEY = AMAZON_ACCESS_KEY_DICT['user']

# file to copy all xml input from the amazon webservice (the content is only
# copyied if a file is specified)
AMAZON_DEBUG_FILE = ""


# ===================================================================
# PATHS AND URLs
# ===================================================================

AMAZON_LOCATIONS = { 'us': 'http://webservices.amazon.com/onca/xml?Service=AWSECommerceService',
                     'uk': 'http://webservices.amazon.co.uk/onca/xml?Service=AWSECommerceService',
		     'de': 'http://webservices.amazon.de/onca/xml?Service=AWSECommerceService',
		     'jp': 'http://webservices.amazon.co.jp/onca/xml?Service=AWSECommerceService',
		     'fr': 'http://webservices.amazon.fr/onca/xml?Service=AWSECommerceService',
		     'ca': 'http://webservices.amazon.ca/onca/xml?Service=AWSECommerceService' }

# ===================================================================
# CORPUS LOCATIONS
# ===================================================================
CORPUS_DIR = '/data/corpus'
AUTOCLASS_SAMPLE = "reuters-10000-multitoken"

BBC_CORPUS       = os_join( CORPUS_DIR, "news.bbc.co.uk" )
BBC_CORPUS_HTML  = os_join( BBC_CORPUS, "html" )
BBC_CORPUS_LOW   = os_join( BBC_CORPUS, "low" )
BBC_CORPUS_RSS   = os_join( BBC_CORPUS, "rss" )
BBC_CORPUS_PRINT = os_join( BBC_CORPUS, "print" )

BBC_CORPUS_TIMEFORMAT = "%a, %d %b %Y %H:%M:%S %Z"

# COMPRESSED VERSION OF THE CORPUS
BBC_CORPUS_COMPRESSED = "data/news.bbc.co.uk.tar.gz"

# geonames
GEO_ENTITY_SEPARATOR = ">"

# facebook
FACEBOOK_API_KEY = "api-key"
FACEBOOK_SECRET_KEY = "secret-key"
FACEBOOK_SESSION_KEY = "session-key"
FACEBOOK_ACCESS_KEY = "access-key"
FACEBOOK_APPLICATION_ID = "application-id"

SNMP_HOST_CFG = {'server'          : 'localhost',
                 'port'            : '162',
                 'community_string': 'public',
                 'oid'             : 'SNMPv2-SMI::enterprises.30538.5000',
                }

SNMP_MODULE_NAME = {}
SNMP_LOG_LEVEL   = { 'ok': '0', 'warning': '1', 'critical': '2' }

# twitter keys .. gila 
TWITTER_CONSUMER_KEY    = ''
TWITTER_CONSUMER_SECRET = ''
TWITTER_ACCESS_TOKEN    = ''
TWITTER_TOKEN_SECRET    = ''

# --------------------------------------------------------------------------
#
#  Overwrite global config based on 
#   a) the system configuraiton and
#   b) the user configuration.
#
# --------------------------------------------------------------------------
if exists( SYS_EWRT_CONF ):
    sys.path.append( dirname(SYS_EWRT_CONF) )
    try:
        from sysconfig import *
    except ImportError:
        warn("No siteconfig in present.")

if exists( USR_EWRT_CONF ):
    sys.path.append( dirname(USR_EWRT_CONF) )
    try:
        from siteconfig import *
    except ImportError:
        warn("Could not finde siteconfig.py in ~/.eWRT")
    
# $Id$



#!/usr/bin/env python

""" an example script querying book reviews from the amazon
    database """

# (C)opyrights 2008 by Albert Weichselbraun <albert@weichselbraun.net>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

MAX_PAGES_TO_PARSE = 10
from eWRT.ws.amazon import AmazonWS, ResultList

def getReview( asin_list ):
	""" returns the reviews for the given asins """
	a=AmazonWS()

	for asin in asin_list:
		result=[]
		for page in xrange(MAX_PAGES_TO_PARSE):
			xmlOutput = a.queryReview( asin, ReviewPage=str(page+1) )
			xmlParser  = ResultList("ItemLookupResponse/Items/Item/CustomerReviews/Review", hunt=("TotalReviewPages",))
			xmlParser.parse( xmlOutput )
			result.extend( xmlParser.xmlResult )

			print("%s: page %s/%s" % (asin, page+1,xmlParser.huntValue.get('TotalReviewPages',0)))
			if page+1 >= int(xmlParser.huntValue.get('TotalReviewPages',0)):
				break


		for review in result:
			review['Summary']=review['Summary'].replace("'", "\\'")
			review['Content']=review['Content'].replace("'", "\\'")
			if not "CustomerId" in review: 
				print("Skipping record\n")
				continue
			print "INSERT INTO Review (asin,customerId,rating,helpfulVotes,totalVotes,date,summary,content) VALUES('%(ASIN)s', '%(CustomerId)s', %(Rating)s, %(HelpfulVotes)s, %(TotalVotes)s, '%(Date)s', '%(Summary)s', '%(Content)s');" % (review)

def exportTopBooks( number  ):
	""" exports the top 'number' books from amazon """

	a = AmazonWS()
	result=[]
	for page in xrange(number/10):
		xmlOutput = a.searchItem( ItemPage=str(page+1) )
		xmlParser = ResultList("ItemSearchResponse/Items/Item")
		xmlParser.parse( xmlOutput )
		result.extend( xmlParser.xmlResult )
	
	for dataSet in result:
		dataSet['Title'] = dataSet['Title'].replace("'", "\\'")
		if len(dataSet['ASIN'])<10: continue
		assert( len(dataSet['ASIN'])==10 )
		print "INSERT INTO Book2 (asin,salesrank,title,detailPageUrl) VALUES ('%(ASIN)s', %(SalesRank)s, '%(Title)s', '%(DetailPageURL)s');" % (dataSet)
	

# retrieve the reviews for the given asins
asin_list = ('0385523416', '1592764053', 'B0013HA824') 
getReview( asin_list )

# exports the 10 top books from amazon
exportTopBooks(10)



#!/usr/bin/env python

'''
Created on Aug 30, 2014

:author: svakulenko
'''

from eWRT.ws.rest import RESTClient
from eWRT.ws import AbstractIterableWebSource


class CustomSearch(AbstractIterableWebSource):

    """wrapper for the Google Custom Search API"""

    NAME = "Google Custom Search"
    ROOT_URL = 'https://www.googleapis.com/customsearch'
    DEFAULT_MAX_RESULTS = 10  # requires only 1 api access
    SUPPORTED_PARAMS = ['command', 'output_format', 'language']
    DEFAULT_COMMAND = 'v1'
    DEFAULT_FORMAT = 'json'
    DEFAULT_START_INDEX = 1
    RESULT_PATH = lambda x: x['items']  # path to the results
    DEFAULT_RESULT_LANGUAGE = 'lang_de'  # lang_en

    MAPPING = {'date': ('valid_from', 'convert_date'),
               'text': ('content', None),
               'title': 'Title',
               }

    def __init__(self, api_key, engine_id, api_url=ROOT_URL):
        """fixes the credentials and initiates the RESTClient"""

        assert(api_key)
        self.api_key = api_key

        assert(engine_id)
        self.engine_id = engine_id

        self.api_url = api_url
        self.client = RESTClient(self.api_url, authentification_method='basic')

    def search_documents(self, search_terms, max_results=DEFAULT_MAX_RESULTS,
                         from_date=None, to_date=None, command=DEFAULT_COMMAND,
                         output_format=DEFAULT_FORMAT,
                         language=DEFAULT_RESULT_LANGUAGE):
        """calls iterator and results' post-processor"""

        fetched = self.invoke_iterator(search_terms, max_results, from_date,
                                       to_date, command, output_format)

        result_path = lambda x: x['items']
        return self.process_output(fetched, result_path)

    def request(self, search_term, current_index,
                max_results=DEFAULT_MAX_RESULTS, from_date=None, to_date=None,
                command=DEFAULT_COMMAND, output_format=DEFAULT_FORMAT,
                language=DEFAULT_RESULT_LANGUAGE):
        """calls Google Custom Search API"""

        parameters = {'q': '"%s"' % search_term,
                      'alt': output_format,
                      'cx': self.engine_id,
                      'key': self.api_key,
                      'lr': language,
                      'num': max_results,
                      'start': current_index}

        # for testing purposes
        # print(current_index, max_results, search_term)

        response = self.client.execute(command, query_parameters=parameters)
        return response

    @classmethod
    def convert_item(cls, item):
        """output convertor: applies a mapping to convert
        the result to the required format
        """

        result = {'url': item['link'],
                  'title': item['title'],
                  }

        return result


class TestCustomSearch(object):

    # provide your google api key for browser applications (from Developers
    # Console)
    my_api_key = 'AIzaSyAlXco-6Bpikl0Ji2H9NEloe4OsL-pUs2g'
    # provide your Custom search engine ID
    my_engine_id = '013438061017685574719:90y0qqxdojg'

    search_terms = ['modul', 'university']

    # test default api call
    def test_default(self):
        cs = CustomSearch(self.my_api_key, self.my_engine_id)
        max_results = cs.DEFAULT_MAX_RESULTS
        results = cs.search_documents(self.search_terms)

        # assert the correct number of the results
        assert len(list(results)) == max_results * len(self.search_terms)

    def test_smaller_max_results(self, max_results=4):
        cs = CustomSearch(self.my_api_key, self.my_engine_id)

        assert max_results < cs.DEFAULT_MAX_RESULTS

        results = cs.search_documents(self.search_terms, max_results)

        # assert the correct number of the results
        assert len(list(results)) == max_results * len(self.search_terms)

    # test several api calls (limit > DEFAULT_MAX_RESULTS)
    def test_larger_max_results(self, max_results=21):
        cs = CustomSearch(self.my_api_key, self.my_engine_id)

        assert max_results > cs.DEFAULT_MAX_RESULTS
        assert max_results % cs.DEFAULT_MAX_RESULTS != 0

        results = cs.search_documents(self.search_terms, max_results)

        # for the testing purposes
        # print(next(results))

        # assert the correct number of the results
        assert len(list(results)) == max_results * len(self.search_terms)

# for the testing purposes
if __name__ == '__main__':
    test = TestCustomSearch()
    test.test_larger_max_results()



#!/usr/bin/env python
"""
   @package eWRT.ws.wikipedia.distance
   wikipedia "click" distance between concepts
"""

# -----------------------------------------------------------------------------------
# (C)opyrights 2010 by Albert Weichselbraun <albert@weichselbraun.net>
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# -----------------------------------------------------------------------------------

__author__   = "albert"
__revision__ = "$Revision: 1 $"

from eWRT.access.db import PostgresqlDb
from eWRT.config import DATABASE_CONNECTION
from nose.plugins.attrib import attr

class WikiDistance(object):
    
    escape_terms = staticmethod( lambda tt:", ".join( ["'%s'" % t.replace("'", "''") for t in tt]) ) 
    
    def __init__(self):
        """ open the database connection """
        dbParam = DATABASE_CONNECTION['wikipedia'].copy()
        dbParam['connect'] = False
        self.db = PostgresqlDb( **dbParam)

    def isSibling(self, t1, t2):
        """ determines whether the given concepts are siblings
            @param[in] t1   first term
            @param[in] t2   second term
            @return True if both terms are siblings
        """
        tt = WikiDistance.escape_terms( (t1,t2) ) 
        with self.db as c:
            q="SELECT * FROM vw_siblings WHERE dname IN (%s) AND sname IN (%s)" % (tt, tt)
            cnt = len(c.query(q))
        return cnt > 0 

    def isSameAs(self, t1, t2):
        """ determines whether the given terms refer to the same concept
            @param[in] t1   first term
            @param[in] t2   second term
            @return True if both terms refer to the same concept
        """
        tt = WikiDistance.escape_terms( (t1,t2) )
        with self.db as c:
            q="SELECT * FROM vw_sameas WHERE dname IN (%s) AND sname IN (%s)" % (tt, tt)
            cnt = len(c.query(q))
        return cnt > 0 
        
        

class TestWikiDistance(object):
    
    def __init__(self):
        self.wd = WikiDistance()
        
    @attr("db")
    def testSameAs(self):
        """ tests the sameAs method """
        assert self.wd.isSameAs("cpu", "central processing unit")
        assert not self.wd.isSameAs("cpu", "desk")

    @attr("db")
    def testIsSibling(self):
        """ tests whether terms are siblings """
        assert self.wd.isSibling("swimming (sport)", "front crawl")
        assert self.wd.isSibling("swimming (sport)", "butterfly stroke")
        assert not self.wd.isSibling("cpu", "front crawl")
 
    @attr("db")
    def testTermPairs(self):
        """ test specific term pairs"""
        assert self.wd.isSibling("design area", "risk") == False

    @staticmethod
    @attr("db")
    def testMultiProcessing():
        """ tests the multi processing capabilities of this module """
        from multiprocessing import Pool
        p = Pool(4)
        res = p.map(p_isSibling, [('cpu', 'desk'), ('cpu', 'central processing unit'),
                                  ('austria', 'carinthia'), ('linux', 'bsd'),
                                  ('microsoft', 'microsoft inc.'), ('anna', 'ana') ]
                                 )
        assert [ same for same, _ in res ] == [ False, True, False, False, True, False ]

    @attr("db")
    def testConnectionLimit(self):
        for _ in xrange(300):
            assert self.wd.isSameAs("design area", "risk") == False


def p_isSibling( concepts ):
    """ 
        @param[in] concepts a tuple containing the two concepts to check

        @remarks
        helper function for the testMultiProcessing test
    """
    assert len(concepts) == 2
    w = WikiDistance()
    return w.isSameAs( *concepts ), w.isSibling( *concepts )




# ----------------------------------------------------------
# AdvaS Advanced Search 
# module for phonetic algorithms
#
# (C) 2002 - 2005 Frank Hofmann, Chemnitz, Germany
# email fh@efho.de
# ----------------------------------------------------------

# changed 2005-01-24

import string
import re

def soundex (term):
	"Return the soundex value to a string argument."

	# Create and compare soundex codes of English words.
	#
	# Soundex is an algorithm that hashes English strings into
	# alpha-numerical value that represents what the word sounds
	# like. For more information on soundex and some notes on the
	# differences in implemenations visit:
	# http://www.bluepoof.com/Soundex/info.html
	#
	# This version modified by Nathan Heagy at Front Logic Inc., to be
	# compatible with php's soundexing and much faster.
	#
	# eAndroid / Nathan Heagy / Jul 29 2000
	# changes by Frank Hofmann / Jan 02 2005

	# generate translation table only once. used to translate into soundex numbers
	#table = string.maketrans('abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ', '0123012002245501262301020201230120022455012623010202')
	table = string.maketrans('ABCDEFGHIJKLMNOPQRSTUVWXYZ', '01230120022455012623010202')

	# check parameter
	if not term:
		return "0000" # could be Z000 for compatibility with other implementations
	# end if

    	# convert into uppercase letters
	term = string.upper(term)
    	first_char = term[0]

	# translate the string into soundex code according to the table above
	term = string.translate(term[1:], table)
	
	# remove all 0s
	term = term.replace("0", "")

    	# remove duplicate numbers in-a-row
    	str2 = first_char
	for x in term:
		if x != str2[-1]:
	   		str2 = str2 + x
		# end if
	# end for

	# pad with zeros
	str2 = str2+"0"*len(str2)

	# take the first four letters
	return_value = str2[:4]

	# return value
	return return_value

def metaphone (term):
	"returns metaphone code for a given string"

	# implementation of the original algorithm from Lawrence Philips
	# extended/rewritten by M. Kuhn
	# improvements with thanks to John Machin <sjmachin@lexicon.net>

	# define return value
	code = ""

	i = 0
	term_length = len(term)

	if (term_length == 0):
		# empty string ?
		return code
	# end if

	# extension #1 (added 2005-01-28)
	# convert to lowercase
	term = term.lower()
	
	# extension #2 (added 2005-01-28)
	# remove all non-english characters, first
	term = re.sub(r'[^a-z]', '', term)
	if len(term) == 0:
		# nothing left
		return code
	# end if
		
	# extension #3 (added 2005-01-24)
	# conflate repeated letters
	firstChar = term[0]
    	str2 = firstChar
	for x in term:
		if x != str2[-1]:
	   		str2 = str2 + x
		# end if
	# end for
	
	# extension #4 (added 2005-01-24)
	# remove any vowels unless a vowel is the first letter
	firstChar = str2[0]
	str3 = firstChar
	for x in str2[1:]:
		if (re.search(r'[^aeiou]', x)):
			str3 = str3 + x
		# end if
	# end for
	
	term = str3
	term_length = len(term)
	if term_length == 0:
		# nothing left
		return code
	# end if
	
	# check for exceptions
	if (term_length > 1):
		# get first two characters
		first_chars = term[0:2]

		# build translation table
		table = {
		    "ae":"e",
		    "gn":"n",
		    "kn":"n",
		    "pn":"n",
		    "wr":"n",
		    "wh":"w"
		}
		
		if first_chars in table.keys():
			term = term[2:]
			code = table[first_chars]
			term_length = len(term)
		# end if
		
	elif (term[0] == "x"):
		term = ""
		code = "s"
		term_length = 0
	# end if

	# define standard translation table
	st_trans = {
		"b":"b",
		"c":"k",
		"d":"t",
		"g":"k",
		"h":"h",
		"k":"k",
		"p":"p",
		"q":"k",
		"s":"s",
		"t":"t",
		"v":"f",
		"w":"w",
		"x":"ks",
		"y":"y",
		"z":"s"
	}

	i = 0
	while (i<term_length):
		# init character to add, init basic patterns
		add_char = ""
		part_n_2 = ""
		part_n_3 = ""
		part_n_4 = ""
		part_c_2 = ""
		part_c_3 = ""

		# extract a number of patterns, if possible
		if (i < (term_length - 1)):
			part_n_2 = term[i:i+2]

			if (i>0):
				part_c_2 = term[i-1:i+1]
				part_c_3 = term[i-1:i+2]
			# end if
		# end if

		if (i < (term_length - 2)):
			part_n_3 = term[i:i+3]
		# end if

		if (i < (term_length - 3)):
			part_n_4 = term[i:i+4]
		# end if

		# use table with conditions for translations
		if (term[i] == "b"):
			add_char = st_trans["b"]
			if (i == (term_length - 1)):
				if (i>0):
					if (term[i-1] == "m"):
						add_char = ""
					# end if
				# end if
			# end if
		elif (term[i] == "c"):
			add_char = st_trans["c"]
			if (part_n_2 == "ch"):
				add_char = "x"
			elif (re.search(r'c[iey]', part_n_2)):
				add_char = "s"
			# end if

			if (part_n_3 == "cia"):
				add_char = "x"
			# end if

			if (re.search(r'sc[iey]', part_c_3)):
				add_char = ""
			# end if

		elif (term[i] == "d"):
			add_char = st_trans["d"]
			if (re.search(r'dg[eyi]', part_n_3)):
				add_char = "j"
			# end if

		elif (term[i] == "g"):
			add_char = st_trans["g"]

			if (part_n_2 == "gh"):
				if (i == (term_length - 2)):
					add_char = ""
				# end if
			elif (re.search(r'gh[aeiouy]', part_n_3)):
				add_char = ""
			elif (part_n_2 == "gn"):
				add_char = ""
			elif (part_n_4 == "gned"):
				add_char = ""
			elif (re.search(r'dg[eyi]',part_c_3)):
				add_char = ""
			elif (part_n_2 == "gi"):
				if (part_c_3 != "ggi"):
					add_char = "j"
				# end if
			elif (part_n_2 == "ge"):
				if (part_c_3 != "gge"):
					add_char = "j"
				# end if
			elif (part_n_2 == "gy"):
				if (part_c_3 != "ggy"):
					add_char = "j"
				# end if
			elif (part_n_2 == "gg"):
				add_char = ""
			# end if
		elif (term[i] == "h"):
			add_char = st_trans["h"]
			if (re.search(r'[aeiouy]h[^aeiouy]', part_c_3)):
				add_char = ""
			elif (re.search(r'[csptg]h', part_c_2)):
				add_char = ""
			# end if
		elif (term[i] == "k"):
			add_char = st_trans["k"]
			if (part_c_2 == "ck"):
				add_char = ""
			# end if
		elif (term[i] == "p"):
			add_char = st_trans["p"]
			if (part_n_2 == "ph"):
				add_char = "f"
			# end if
		elif (term[i] == "q"):
			add_char = st_trans["q"]
		elif (term[i] == "s"):
			add_char = st_trans["s"]
			if (part_n_2 == "sh"):
				add_char = "x"
			# end if

			if (re.search(r'si[ao]', part_n_3)):
				add_char = "x"
			# end if
		elif (term[i] == "t"):
			add_char = st_trans["t"]
			if (part_n_2 == "th"):
				add_char = "0"
			# end if

			if (re.search(r'ti[ao]', part_n_3)):
				add_char = "x"
			# end if
		elif (term[i] == "v"):
			add_char = st_trans["v"]
		elif (term[i] == "w"):
			add_char = st_trans["w"]
			if (re.search(r'w[^aeiouy]', part_n_2)):
				add_char = ""
			# end if
		elif (term[i] == "x"):
			add_char = st_trans["x"]
		elif (term[i] == "y"):
			add_char = st_trans["y"]
		elif (term[i] == "z"):
			add_char = st_trans["z"]
		else:
			# alternative
			add_char = term[i]
		# end if

		code = code + add_char
		i += 1
	# end while

	# return metaphone code
	return code

def nysiis (term):
	"returns New York State Identification and Intelligence Algorithm (NYSIIS) code for the given term"

	code = ""

	i = 0
	term_length = len(term)

	if (term_length == 0):
		# empty string ?
		return code
	# end if

	# build translation table for the first characters
	table = {
		"mac":"mcc",
		"ph":"ff",
		"kn":"nn",
		"pf":"ff",
		"k":"c",
		"sch":"sss"
	}

	for table_entry in table.keys():
		table_value = table[table_entry]	# get table value
		table_value_len = len(table_value)	# calculate its length
		first_chars = term[0:table_value_len]
		if (first_chars == table_entry):
			term = table_value + term[table_value_len:]
			break
		# end if
	# end for

	# build translation table for the last characters
	table = {
		"ee":"y",
		"ie":"y",
		"dt":"d",
		"rt":"d",
		"rd":"d",
		"nt":"d",
		"nd":"d",
	}

	for table_entry in table.keys():
		table_value = table[table_entry]	# get table value
		table_entry_len = len(table_entry)	# calculate its length
		last_chars = term[(0 - table_entry_len):]
		#print last_chars, ", ", table_entry, ", ", table_value
		if (last_chars == table_entry):
			term = term[:(0 - table_value_len + 1)] + table_value
			break
		# end if
	# end for

	# initialize code
	code = term

	# transform ev->af
	code = re.sub(r'ev', r'af', code)

	# transform a,e,i,o,u->a
	code = re.sub(r'[aeiouy]', r'a', code)
	
	# transform q->g
	code = re.sub(r'q', r'g', code)
	
	# transform z->s
	code = re.sub(r'z', r's', code)

	# transform m->n
	code = re.sub(r'm', r'n', code)

	# transform kn->n
	code = re.sub(r'kn', r'n', code)

	# transform k->c
	code = re.sub(r'k', r'c', code)

	# transform sch->sss
	code = re.sub(r'sch', r'sss', code)

	# transform ph->ff
	code = re.sub(r'ph', r'ff', code)

	# transform h-> if previous or next is nonvowel -> previous
	occur = re.findall(r'([a-z]{0,1}?)h([a-z]{0,1}?)', code)
	#print occur
	for occur_group in occur:
		occur_item_previous = occur_group[0]
		occur_item_next = occur_group[1]

		if ((re.match(r'[^aeiouy]', occur_item_previous)) or (re.match(r'[^aeiouy]', occur_item_next))):
			if (occur_item_previous != ""):
				# make substitution
				code = re.sub (occur_item_previous + "h", occur_item_previous * 2, code, 1)
			# end if
		# end if
	# end for
	
	# transform w-> if previous is vowel -> previous
	occur = re.findall(r'([aeiouy]{1}?)w', code)
	#print occur
	for occur_group in occur:
		occur_item_previous = occur_group[0]
		# make substitution
		code = re.sub (occur_item_previous + "w", occur_item_previous * 2, code, 1)
	# end for
	
	# check last character
	# -s, remove
	code = re.sub (r's$', r'', code)
	# -ay, replace by -y
	code = re.sub (r'ay$', r'y', code)
	# -a, remove
	code = re.sub (r'a$', r'', code)
	
	# return nysiis code
	return code

def caverphone (term):
	"returns the language key using the caverphone algorithm 2.0"

	# Developed at the University of Otago, New Zealand.
	# Project: Caversham Project (http://caversham.otago.ac.nz)
	# Developer: David Hood, University of Otago, New Zealand
	# Contact: caversham@otago.ac.nz
	# Project Technical Paper: http://caversham.otago.ac.nz/files/working/ctp150804.pdf
	# Version 2.0 (2004-08-15)

	code = ""

	i = 0
	term_length = len(term)

	if (term_length == 0):
		# empty string ?
		return code
	# end if

	# convert to lowercase
	code = term.lower()

	# remove anything not in the standard alphabet (a-z)
	code = re.sub(r'[^a-z]', '', code)

	# remove final e
	if code.endswith("e"):
		code = code[:-1]

	# if the name starts with cough, rough, tough, enough or trough -> cou2f (rou2f, tou2f, enou2f, trough)
	code = re.sub(r'^([crt]|(en)|(tr))ough', r'\1ou2f', code)

	# if the name starts with gn -> 2n
	code = re.sub(r'^gn', r'2n', code)

	# if the name ends with mb -> m2
	code = re.sub(r'mb$', r'm2', code)

	# replace cq -> 2q
	code = re.sub(r'cq', r'2q', code)
	
	# replace c[i,e,y] -> s[i,e,y]
	code = re.sub(r'c([iey])', r's\1', code)
	
	# replace tch -> 2ch
	code = re.sub(r'tch', r'2ch', code)
	
	# replace c,q,x -> k
	code = re.sub(r'[cqx]', r'k', code)
	
	# replace v -> f
	code = re.sub(r'v', r'f', code)
	
	# replace dg -> 2g
	code = re.sub(r'dg', r'2g', code)
	
	# replace ti[o,a] -> si[o,a]
	code = re.sub(r'ti([oa])', r'si\1', code)
	
	# replace d -> t
	code = re.sub(r'd', r't', code)
	
	# replace ph -> fh
	code = re.sub(r'ph', r'fh', code)

	# replace b -> p
	code = re.sub(r'b', r'p', code)
	
	# replace sh -> s2
	code = re.sub(r'sh', r's2', code)
	
	# replace z -> s
	code = re.sub(r'z', r's', code)

	# replace initial vowel [aeiou] -> A
	code = re.sub(r'^[aeiou]', r'A', code)

	# replace all other vowels [aeiou] -> 3
	code = re.sub(r'[aeiou]', r'3', code)

	# replace j -> y
	code = re.sub(r'j', r'y', code)

	# replace an initial y3 -> Y3
	code = re.sub(r'^y3', r'Y3', code)
	
	# replace an initial y -> A
	code = re.sub(r'^y', r'A', code)

	# replace y -> 3
	code = re.sub(r'y', r'3', code)
	
	# replace 3gh3 -> 3kh3
	code = re.sub(r'3gh3', r'3kh3', code)
	
	# replace gh -> 22
	code = re.sub(r'gh', r'22', code)

	# replace g -> k
	code = re.sub(r'g', r'k', code)

	# replace groups of s,t,p,k,f,m,n by its single, upper-case equivalent
	for single_letter in ["s", "t", "p", "k", "f", "m", "n"]:
		otherParts = re.split(single_letter + "+", code)
		code = string.join(otherParts, string.upper(single_letter))
	
	# replace w[3,h3] by W[3,h3]
	code = re.sub(r'w(h?3)', r'W\1', code)

	# replace final w with 3
	code = re.sub(r'w$', r'3', code)

	# replace w -> 2
	code = re.sub(r'w', r'2', code)

	# replace h at the beginning with an A
	code = re.sub(r'^h', r'A', code)

	# replace all other occurrences of h with a 2
	code = re.sub(r'h', r'2', code)

	# replace r3 with R3
	code = re.sub(r'r3', r'R3', code)

	# replace final r -> 3
	code = re.sub(r'r$', r'3', code)

	# replace r with 2
	code = re.sub(r'r', r'2', code)

	# replace l3 with L3
	code = re.sub(r'l3', r'L3', code)
	
	# replace final l -> 3
	code = re.sub(r'l$', r'3', code)
	
	# replace l with 2
	code = re.sub(r'l', r'2', code)

	# remove all 2's
	code = re.sub(r'2', r'', code)

	# replace the final 3 -> A
	code = re.sub(r'3$', r'A', code)
	
	# remove all 3's
	code = re.sub(r'3', r'', code)

	# extend the code by 10 '1' (one)
	code += '1' * 10
	
	# take the first 10 characters
	caverphoneCode = code[:10]
	
	# return caverphone code
	return caverphoneCode




#!/usr/bin/env python

from unittest import TestCase

from eWRT.ws.conceptnet.lookup_result import LookupResult
from eWRT.ws.conceptnet.util import ground_term

def test_lookup_result():
    l = LookupResult("dog")

def test_concept_grounding():
    concept = ground_term('battery', ['life', 'extensive', 'too', 'short', 'electricity'])
    print "--_"
    print concept
    print "--__"
    aha



#!/usr/bin/env python

from json import dumps
from eWRT.ws.conceptnet import Result

def test_get_vsm():
    json_string = dumps(eval("""{'edges': [{u'features': [u'/c/en/look_at /r/DerivedFrom -', u'/c/en/look_at - /c/en/look/v', u'- /r/DerivedFrom /c/en/look/v'], u'weight': 1.5, u'text': [u'look', u'look at'], u'dataset': u'/d/wiktionary/en/en', u'sources': [u'/s/rule/wiktioary_monolingual_definitions', u'/s/web/en.wiktionary.org'], u'id': u'/e/8d03305cbe5d3aa19cf4e35cf56b9cfa4578c7ce', u'surfaceText': u'[[look at]] DerivedFrom [[look]]', u'endLemmas': u'look', u'end': u'/c/en/look/v', u'license': u'/l/CC/By-SA', u'startLemmas': u'look at', u'uri': u'/a/[/r/DerivedFrom/,/c/en/look_at/,/c/en/look/v/]', u'rel': u'/r/DerivedFrom', u'start': u'/c/en/look_at', u'score': 9.476239, u'context': u'/ctx/all', u'timestamp': u'2013-03-05T06:36:23.081Z', u'nodes': [u'/c/en/look/v', u'/c/en/look_at', u'/r/DerivedFrom']}]}"""))

    r = Result(json_string)
    assert len(r.get_vsm(stopword_list=[])) == 2
    assert 'look' in r.get_vsm(stopword_list=[])
    assert 'at' in r.get_vsm(stopword_list=[])

    assert len(r.get_vsm(stopword_list=['look', 'at'])) == 0




#!/usr/bin/env python

## create a iterable ResultSet for storing values of the Webservices 
class ResultSet:

    ## Constructor
    # @parameter id (optional) integer
    # @parameter name (optional) String
    # @parameter content (optional) a Result or ResultSet
    def __init__(self, id=None, name=None, content=None): 
        self.id = id
        self.name = name
        self.setContent(content) 
        self.idx = 0 

    ## returns the next item in the ResultSet
    # @result or ResultSet
    # @return content = Result, ResultSet 
    def next(self): 
        #if self.idx > self.content.count:
        if self.idx >= len(self.content): 
            self.refresh()
            raise StopIteration 
        result = self.content[self.idx]
        self.idx += 1 
        return result
    ## function for the iterator
    def __iter__(self): 
        return self 

    ## sets the pointer of the index to 0
    def refresh(self): 
        self.idx = 0 

    ## sets the content to the new content
    # overrides the content 
    # @parameter content = Result, ResultSet 
    def setContent(self, content):
        if content is None:
            self.content = []
        else:
            self.content = content

    ## adds new content to the existing one
    # @parameter newContent = Result or ResultSet
    def addContent(self, newContent):
        self.content.append(newContent)
        """ todo: needs to be implemented """

    ## return the ID of the ResultSet
    # @return ID integer
    def getId(self):
        return self.id

    ## return the Name of the ResultSet
    # @return Name String
    def getName(self):
        return self.name

    ## prints the items stored in the ResultSet
    @staticmethod
    def printRS(resultSet, filler = 0):
        print resultSet.getName(),':'

        for result in resultSet:
            if cmp(result.__class__.__name__, 'ResultSet') == 0:
                # filler = filler + 1
                ResultSet.printRS(result, filler)
            else:
                print result.getId(), result.getName()


if __name__ == "__main__":
    
    from Result import Result

    a = Result(1, 'foobar1')
    b = Result(2, 'foobar2')
    c = Result(3, 'foobar3')
    d = Result(4, 'foobar4')
    e = Result(5, 'foobar5')
    f = Result(6, 'foobar6')
    g = Result(7, 'foobar7')
    h = Result(8, 'foobar8')
    i = Result(9, 'foobar9')

    content = [a, b, c, d]
    rs = ResultSet(1, 'RS1')
    rs.addContent(a)
    rs.addContent(b)
    rs.addContent(c)
    rs.addContent(d)

    rs2 = ResultSet(1, 'RS2')
    rs2.addContent(rs)
    rs2.addContent(e)
    rs2.addContent(f)

    rs2.printRS(rs2)



#!/usr/bin/env python

from eWRT.access.http import Retrieve
from eWRT.ws.TagInfoService import TagInfoService
import re

class Twitter(TagInfoService):

    TWITTER_SEARCH_URL = 'http://search.twitter.com/search.json?q=&tag=%s&lang=all&rpp=100'
    RE_FIND_TAGS = re.compile('#(\w+)', re.IGNORECASE|re.DOTALL)

    @staticmethod
    def getRelatedTags( tags ):
        """ fetches the related tags for the given tags
            @param list of tags
            @return dictionary of related tags with count
        """

        if type(tags).__name__ == 'str':
            url = Twitter.TWITTER_SEARCH_URL % tags
        else:   
            url = Twitter.TWITTER_SEARCH_URL % "+".join(tags)

        f = Retrieve(Twitter.__name__).open(url)

        # convert json into dict and remove null values with ""
        search_results = eval(re.sub('null', '""', f.read()))
        found_tags = []
        related_tags = {}

        for result in search_results['results']:
            found_tags.extend(Twitter.RE_FIND_TAGS.findall( result['text']))

        for tag in found_tags:
            related_tags[tag.lower()] = related_tags.get(tag.lower(), 0) + 1

        # todo: sort

        return related_tags
    
    @staticmethod
    def getDailyTrends():
        raise NotImplementedError


class TwitterTest( object ):

    TWITTER_TEST_TAGS = [ 'linux', ('linux', 'debian') ]

    def testUrlInfo(self):
        for tag in self.TWITTER_TEST_TAGS:
            relTags = Twitter.getRelatedTags(tag)
            print 'related to %s are %s.'% (tag, relTags )




#!/usr/bin/env python

## Result is an item of ResultSet
class Result:

    ## constructor 
    # @parameter id, name
    def __init__(self, id, name):
        self.id = id
        self.name = name

    ## get the ID of Result
    # @return Id
    def getId(self):
        return self.id

    ## get the Name of the Result
    # @return Name
    def getName(self):
        return self.name

    def getAttributes(self):
        """ todo: return attributes of Result """




#!/usr/bin/env python

""" @package eWRT.util.profile
    google like profiling :)

    @warning
    this library is still a draft and might change considerable
    
"""

# (C)opyrights 2008-2009 by Albert Weichselbraun <albert@weichselbraun.net>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

__author__    = "Albert Weichselbraun"
__revision__  = "$Id$"
__copyright__ = "GPL"

from StringIO import StringIO
import cProfile, pstats

def profile(fn, logfile="profile.awi"):
    """ profile function """
    prof = cProfile.Profile()
    
    import __main__
    if __main__.__file__.endswith("nosetests"):
        prof = prof.runctx("%s()" % fn.__name__, globals(), locals())
    else:
        prof = prof.runctx("%s()" % fn.__name__, __main__.__dict__, __main__.__dict__ )

    stream = StringIO()
    stats = pstats.Stats(prof, stream=stream)
    stats.sort_stats("time")  # Or cumulative
    stats.print_stats(80)  # 80 = how many to print
    # The rest is optional.
    stats.print_callees()
    stats.print_callers()
    open(logfile,"w").write( stream.getvalue() )

def __testFunction():
    for a in xrange(1000):
        for b in range(1000):
            c = a * b
           
    return c

def testProfile():
    profile(__testFunction)




#!/usr/bin/env python

""" @package eWRT.ws.amazon
    provides access to amazon data """

# (C)opyrights 2008 by Albert Weichselbraun <albert@weichselbraun.net>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.



__revision__ = "$Revision$"


import time
import logging

from urllib import quote
from xml.parsers.expat import ParserCreate
from eWRT.access.http import Retrieve
from eWRT.config import AMAZON_ACCESS_KEY, AMAZON_LOCATIONS, AMAZON_DEBUG_FILE

from time import strftime

import base64
import hashlib
import hmac
import urllib

# time to wait after an error in seconds
ERROR_SLEEP_TIME = 30

BROWSE_NODE_ID = { 'book' : '283155',
                   'dvd'  : '130',
                   'music': '5174'
                 }

class ResultList:
    """ converts xml results into a list of dictionaries """

    def __init__(self, targetPath, hunt=()):

        assert(isinstance(targetPath, str))

        self.p = ParserCreate()
        self.p.StartElementHandler = self.startElement
        self.p.EndElementHandler = self.endElement
        self.p.CharacterDataHandler = self.charData

        self.xmlResult = []
        self.targetPath = targetPath
        self.path = []

        self.hunt = hunt
        self.huntValue = {}


    def startElement(self, element, attrs):
        """ start Element handler """
        self.path.append(element)
        if self.targetPath == "/".join(self.path):
            self.xmlResult.append({})
        

    def charData(self, data):
        """ adds data to the dictionary """
        currentElement = self.path[-1] 
        if self.targetPath in "/".join(self.path):
                    try:
                        self.xmlResult[-1][ currentElement ] = self.xmlResult[-1].get(currentElement, "") + data.encode("utf8")
                    except IndexError:
                        raise "Given Search Path '%s' is not a Sub-Path of '%s'\n" % ("/".join(self.path), self.targetPath)
                
        if currentElement in self.hunt:
            self.huntValue[currentElement] = data.encode("utf8")
            

    def endElement(self, element):
        assert(self.path)
        self.path.pop()


    def parse(self, data):
        """ parses xml data """
        self.p.Parse(data)


    def parseNew(self, data):
        self.path = []
        self.p.Parse(data)


    def test(self):
        self.parse(open("a.xml").read())
        print self.xmlResult



class AmazonWS:
    """ This class provides low level amazon web service access """
    
    def __init__(self, location='us', key=None):
        """ init """
        assert (location in AMAZON_LOCATIONS)
        self.retrieve = Retrieve(self.__class__.__name__)
        self.wsBase = AMAZON_LOCATIONS[ location ]
        self.accessKey = key or AMAZON_ACCESS_KEY
        self.amazon_url = AmazonUrl()


    def generateWsUrl(self, arguments):
        """ generates a valid amazon webservice request url """
        argList = [ "%s&SubscriptionId=%s" % (self.wsBase, self.accessKey) ] + [ "%s=%s" % (k, quote(v)) for k, v in arguments.items() ]
        return "&".join(argList)

    def generateSignedWsUrl(self, **arguments):
        """ generates a valid amazon webservice request url """
        #argList = [ "%s&SubscriptionId=%s" % (self.wsBase, self.accessKey) ] + [ "%s=%s" % (k,quote(v)) for k,v in arguments.items() ]
        #return "&".join(argList)
        return self.amazon_url.get_request_url(arguments)


    def query(self, arguments):
        """ retrieves a result from amazon webservice """
        url = self.generateWsUrl(arguments)

        done = False
        while not done:
            try:
                f = self.retrieve.open(url)
                res = f.read()
                self._write_debug_data(res)
                f.close()
                done = True
            except ValueError:
                logging.warning("Exception webservice query - waiting for %d seconds...\n" % ERROR_SLEEP_TIME)
                time.sleep(ERROR_SLEEP_TIME)
        return res

        @staticmethod
        def _write_debug_data(data):
            """ writes the given data to the debug file, if specified """
            if not AMAZON_DEBUG_FILE:
                return

            d = open(AMAZON_DEBUG_FILE, "a")
            d.write(data)
            d.close()



    def searchItem(self, searchIndex='Books', **param):
        """ searches an item in the amazon product repository """
        arguments = { 'Operation'    : 'ItemSearch',
                      'SearchIndex'  : searchIndex,
                  'BrowseNode'   : '1000',
                  'Sort'         : 'salesrank',
                  'ResponseGroup': 'SalesRank,Small'}

        arguments.update(param)
        return self.query(arguments)


    def queryReview(self, itemId, **param):
        """ queries customers reviews to the selected Item """
        arguments = { 'Operation' : 'ItemLookup',
                      'ResponseGroup': 'Reviews',
                      'ItemId'    : itemId }
        arguments.update(param)
        return self.query(arguments)


        def newReleases(self, **param):
            """ returns a list of asins of new releases """
            arguments = { 'Operation'    : 'BrowseNodeLookup',
                          'ResponseGroup': 'NewReleases',
                          'Marketplace'  : 'us' }

            arguments.update(param)
            return self.query(arguments)


        def itemAttributes(self, item_id, **param):
            """ returns all item attribues """
            arguments = {'Operation'    : 'ItemLookup',
                         'ItemId'       : item_id,
                         'IdType'       : 'ASIN',
                         'ResponseGroup': 'ItemAttributes,SalesRank' }
            arguments.update(param)
            return self.query(arguments)


AWS_ACCESS_KEY_ID = {'AWSAccessKeyId': AMAZON_ACCESS_KEY}

class AmazonUrl:

    def timestamp(self):
        """ determine the current timestamp """
        current_timestamp = strftime('%Y-%m-%dT%H:%M:%SZ')
        return {'Timestamp': current_timestamp}

    def get_signature(self, msg):
        """ apply hmac with sha256 to the msg using key """
        hmac_sha256_alg = hmac.new(self.access_key, msg, hashlib.sha256)
        signature = hmac_sha256_alg.digest()
        signature = base64.encodestring(signature)
        return signature

    def get_msgs(self, params):
        """ determine the sig_msg (i.e. the message to be hmac encoded) and the
        url_msg (i.e. the message to be sent to amazon) """

        GET = 'GET'
        ECS = 'ecs.amazonaws.com'
        ONCA = '/onca/xml'
        
        params.update(self.timestamp())
        params.update(AWS_ACCESS_KEY_ID)
        url_encoded = urllib.urlencode(params)

        msg = [entry for entry in url_encoded.split('&')]
        msg.sort() # amazon also calculates signature from sorted entries
        msg = '&'.join(msg)

        sig_msg = '\n'.join([GET, ECS, ONCA, msg])
        url_msg = '%s%s%s?%s' % ('http://', ECS, ONCA, msg)
        return url_msg, sig_msg

    def get_request_url(self, params):
        """ generate a url containing signature and timestamp with the given
        parameters """
        url_msg, sig_msg = self.get_msgs(params)
        signature = self.get_signature(sig_msg)
        request_url = "%s&%s" % (url_msg, urllib.urlencode({ 'Signature': signature }))
        return request_url.replace('%0A', '')

if __name__ == "__main__":
    url_generator = AmazonWS()
    url = url_generator.generateSignedWsUrl(Operation='BrowseNodeLookup', Service='AWSECommerceService', ResponseGroup='NewReleases', Marketplace='us', BrowseNodeId='281052')
    print url



#!/usr/bin/env python

"""
::package eWRT.ws.conceptnet
Access to conceptnet data structures using its REST interface

::author: Albert Weichselbraun <albert.weichselbraun@htwchur.ch>
"""

from collections import namedtuple, Counter
from itertools import chain
from json import loads
from os.path import exists
from cPickle import load, dump

from eWRT.access.http import Retrieve
from eWRT.util.cache import DiskCached
from eWRT.ws.conceptnet.node import Node
from eWRT.ws.conceptnet.edge import Edge

CONCEPTNET_BASE_URL = 'http://conceptnet5.media.mit.edu/data/5.1'
CLEANUP_TRANSLATION_MAP = {'!': None, '.': None, '?': None, 
                           '"': None, "'": None}

Concept  = namedtuple("Concept", "language concept_name pos sense")
tokenize = lambda sentence: sentence.translate(CLEANUP_TRANSLATION_MAP).split(" ")

@DiskCached(".conceptnet-query-cache")
def retrieve_conceptnet_query_result(query):
    ''' ::param url: the url to retrieve
        ::return: the json response to the given conceptnet query
    '''
    with Retrieve(__name__) as r:
        c = r.open(query, retry=3)
        return c.read()


class Result(object):
    ''' The result of a ConceptNet query '''

    RELEVANT_VSM_ATTRIBUTES = ('endLemmas', 'startLemmas', 'text', )

    def __init__(self, json_string, min_score=1.0):
        '''
        ::param json_string: the conceptnet json string
        ::param min_score: minimum confidence score required for
                           an edge to be included.
        '''
        self.edges = [Edge(edge_dict) for edge_dict in loads(json_string)['edges']
                      if edge_dict['score'] >= min_score]

        #edge_types = load(open("known-edge-types.awi")) if exists("known-edge-types.awi") else set()
        #edge_types.update([ e['rel'] for e in self.edges ])
        #dump(edge_types, open("known-edge-types.awi","w"))

    def apply_edge_filter(self, filter_list):
        '''
        Applies the given filter to the result object
        ::param filter_list: a list containing keys and 
             potential values for these keys. The entry is filtered
             if it does not match any of the given key value pairs.
             (e.g. [('rel', '/r/isA'), ('rel', '/r/HasContext'), ...])
        '''
        self.edges = [edge for edge in self.edges if 
                      [True for key, value in filter_list
                       if edge[key] == value]]

    def apply_language_filter(self, valid_languages):
        '''
        Removes all edges where either the start or end node
        does not matches the list of valid languages
        ::param valid_languages: list of valid languages
        '''
        self.edges = [edge for edge in self.edges if
                      Node.lang(edge['start']) in valid_languages and
                      Node.lang(edge['end']) in valid_languages]

    def get_concept(self, filter_url=None, include_subconcepts=False):
        ''' ::param filter_url: an optional filter_url for the concepts to extract 
            ::param include_subconcepts: whether to include subconcepts, if filtering
                       is enabled.
                       (e.g. '/c/en/battery/n/the_battery_used_to_heat_the_filaments_of_a_vacuum_tube'
                             for '/c/en/battery')
        '''
        matches_filter = lambda url: url.startswith(filter_url) if \
                            include_subconcepts else lambda url: url==filter_url

        return [Node(url, self.edges) for url in chain(
                [e['start'] for e in self.edges],
                [e['end'] for e in self.edges]) 
                if not filter_url or matches_filter(url)]

    def get_senses(self):
        ''' ::return: the set of senses present in the result '''
        return set(self.get_concept())

    def get_vsm(self, stopword_list):
        ''' 
        ::param stopword_list: an optional stopword_list to apply
        ::return: a counter object with all words and their respective
                  counts for usage in a vector space model.
        '''
        vsm = Counter()

        for attr_value in (edge[attr] for edge in self.edges for attr 
           in self.RELEVANT_VSM_ATTRIBUTES 
           if attr in edge):
            
            # handle list values
            if isinstance(attr_value, list):
                attr_value = chain(*map(tokenize, attr_value))
                vsm.update(attr_value)
            else:
                vsm.update( tokenize(attr_value) )

        # apply stopword list
        map(vsm.pop, [ s for s in stopword_list if s in vsm])
        return vsm









    



#!/usr/bin/env python

""" @package eWRT.util.async
    asynchronous procedure calls 

    @warning
    this library is still a draft and might change considerable
    
"""


# (C)opyrights 2008-2010 by Albert Weichselbraun <albert@weichselbraun.net>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

__author__    = "Albert Weichselbraun"
__revision__  = "$Id$"
__copyright__ = "GPL"

from eWRT.util.cache import DiskCache
from shutil import rmtree
from os.path import join, exists
from cPickle import load, UnpicklingError
import time
from subprocess import Popen
from glob import glob
import os
import gzip

try:
    import hashlib
    HASH = hashlib.sha1
except ImportError:
    import sha
    HASH = sha.sha


class Async(DiskCache):
    """ Asynchronous Call Handling """

    def __init__(self, cache_dir, cache_nesting_level=0, cache_file_suffix="", max_processes=8, debug_dir=None):
        """ initializes the Cache object 
            @param[in] cache_dir the cache base directory
            @param[in] cache_nesting_level optional number of nesting level (0)
            @param[in] cache_file_suffix optional suffix for cache files
            @param[in] max_processes maximum number of parallel processes
            @param[in] optional debug directory where stdout and stderr of the processes gets saved
        """

        self.cache_dir           = cache_dir
        self.cache_file_suffix   = cache_file_suffix
        self.cache_nesting_level = cache_nesting_level
        self.max_processes       = max_processes
        self.cur_processes       = []
        self.debug_dir           = debug_dir

    def getPostHashfile(self, cmd ):
        """ returns an identifier representing the object which is compatible 
            to the identifiers returned by the eWRT.util.cache.* classes. """
        args = ( tuple(cmd[1:]), ())  # required to produce the same hash as DiskCache's fetch method
        return self._get_fname( DiskCache.getObjectId( args ) ) 
        
   
    def post(self, cmd):
        """ checks whether the given command is already cached and calls
            the command otherwise.
            @param[in] cmdline command to call
            @returns the hash required to fetch this object
        """
        cache_file = self.getPostHashfile( cmd )  
        # print "I will return %s for %s." % (cache_file, " ".join(cmd) )

        # try to fetch the object from the cache
        if exists(cache_file):
            try:
                load(open(cache_file))
                return cache_file
            except (EOFError, UnpicklingError):
                pass

        self._execute(cmd)
        return cache_file

    def has_processes_limit_reached(self):
        """ closes finished processes and verifies whether we have already 
            reached the maximum number of processes """
        # verify whether all registered processes are still running
        for pObj in self.cur_processes:
            if pObj.poll()!=None:
               self.cur_processes.remove( pObj )

        pids = [ str(pObj.pid) for pObj in self.cur_processes ]
        return len(self.cur_processes) >= self.max_processes

    def _execute(self, cmd):
        while self.has_processes_limit_reached():
            time.sleep(5)

        if self.debug_dir:
            fname_base = join(self.debug_dir, str(time.time()) )
            stdout = open(fname_base+".out", "w")
            stderr = open(fname_base+".err", "w")
            pObj = Popen( cmd, stdout=stdout, stderr=stderr )
            os.rename( fname_base+".out", join(self.debug_dir, "debug_%d.out" % pObj.pid ))
            os.rename( fname_base+".err", join(self.debug_dir, "debug_%d.err" % pObj.pid ))
        else:
            pObj = Popen( cmd )

        # set stdout and stderr to non blocking because otherwise
        # the process will block after a limit of 64k has been reached
        # (see http://bytes.com/topic/python/answers/741393-spawning-process-subprocess)
        # fcntl(stdout, F_SETFL, fcntl(stdout, F_GETFL) | os.O_NONBLOCK)
        # fcntl(stdin, F_SETFL, fcntl(stdin, F_GETFL) | os.O_NONBLOCK)

        self.cur_processes.append( pObj )


    def fetch(self, cache_file):
        self.has_processes_limit_reached()
        while True:
            if exists(cache_file):
                try: 
                    return load( gzip.open(cache_file))
                except (EOFError, UnpicklingError) as e:
                    print "Error opening %s" % cache_file, e
                    pass

            time.sleep(10)


class TestAsync(object):
    """ unittests covering the class async """

    TEST_CACHE_DIR = "./.test-async"

    def setUp(self):
        self._delCacheDir()

    def tearDown(self):
        self._delCacheDir()

    @staticmethod
    def _delCacheDir():
        if exists( TestAsync.TEST_CACHE_DIR ):
            rmtree( TestAsync.TEST_CACHE_DIR )

    def testMaxProcessLimit(self):
        """ tests the max process limit """
        async = Async(self.TEST_CACHE_DIR, max_processes=1)
        for x in xrange(2):
            async.post( [ "/bin/sleep", str(x+1) ] )

        assert async.has_processes_limit_reached() == True

        time.sleep(2)
        flag = async.has_processes_limit_reached()
        print flag, [ p.pid for p in async.cur_processes ]
        assert flag  == False

    def testDebugMode(self):
        """ tests the debug mode """
        async = Async(self.TEST_CACHE_DIR, max_processes=1, debug_dir=self.TEST_CACHE_DIR)
        for x in xrange(2):
            async.post( ["/bin/echo", "hallo"] )

        print glob( join(self.TEST_CACHE_DIR, "debug*") )
        assert len( glob( join(self.TEST_CACHE_DIR, "debug*.out") )  ) == 2
        assert len( glob( join(self.TEST_CACHE_DIR, "debug*.err") )  ) == 2


# $Id$



#!/usr/bin/env python

"""
@package eWRT.ws.rss
@author: Albert Weichselbraun <albert@weblyzard.com>

Retrieves RSS Feeds and the corresponding Web page.
"""

from datetime import datetime
from time import mktime
import feedparser
from eWRT.access.http import Retrieve
from unittest import main

HTTP_FETCH_DELAY = 0

def parse(url, last_modified=None):
    """ 
    Parses the given RSS Feed an returns all articles and the content of
    the page referenced in the <link> tag.
    
    @param url: the url of the rss feed
    @param last_modified: a datetime object that specifies the last time the
                          feed has been queried the last time (only newer 
                          entries are returned).  
    """
    feed = feedparser.parse(url, modified=last_modified)
    retrieve = Retrieve("rss", HTTP_FETCH_DELAY)
    
    result = []
    for item in feed['items']:
        if datetime.fromtimestamp(
                mktime(item['updated_parsed'])) > last_modified:
            item['content'] = retrieve.open(item['link']).read()
            result.append(item)

    return result



def parse_test():
    """ Test the rss module. """
    TEST_URL = "http://kurier.at/rss/channel_startseite_rss.xml"
    last_modified = datetime(2012,8,3,21,50,00)
    for a in parse(TEST_URL, last_modified):
        print a['link'], a['date']
        print len(a['content'])


from eWRT.config import GOOGLE_USER, GOOGLE_PASS

try:
    from eWRTlibs.googleTrends.pyGTrends import pyGTrends
    LOADED = True
except ImportError:
    from warnings import warn
    from sys import exit
    warn("This module requires google Trends ")
    LOADED = False
    
class GoogleTrends(object):
    ''' ## Documentation for the Class Google Trends
        fetches the Google trends for a set of keywords '''

    
    def __init__(self, user=GOOGLE_USER, password=GOOGLE_PASS):
        ''' Constructor establishes the connection
            @param user
            @param password
        '''
        self.connector = pyGTrends(user, password)
        
 
    def getTrends(self, keywords, date=None, section=None):
        ''' getTrends fetches the trends
            @param keywords
            @param date to filter 
            @param section
            @return trends 
        '''
        if date == None:
            self.connector.download_report(keywords)
        else:
            self.connector.download_report(keywords, date)

        print self.connector.header_dictionary

        if section == None:
            trends = self.connector.csv()
        else:
            trends = self.connector.csv()

        return trends

if __name__ == "__main__":
    a = GoogleTrends()
    print a.getTrends(('Apple', 'Microsoft'))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

""" install eWRT :) """
from setuptools import setup, find_packages

from sys import exit

setup(
      ###########################################
      ## Metadata
      name="eWRT",
      version="0.9.1.10",
      description='eWRT',
      author='Albert Weichselbraun, Heinz Lang, Gerhard Wohlgenannt, Johannes Duong, Norman Süsstrunk, Daniel Streiff',
      author_email='albert@weblyzard.com, lang@weblyzard.com, wohlg@ai.wu.ac.at, johannes.duong@wu.ac.at, norman.suestrunk@htwchur.ch, daniel.streiff@htwchur.ch',
      url='http://www.weblyzard.com/ewrt/',
      license="GPL3",
      package_dir={'': 'src'},

      ###########################################
      ## Run unittests
      test_suite='nose.collector',

      ###########################################
      ## Scripts
      scripts=['src/eWRT/input/corpus/reuters/reuters.py' ],

      ###########################################
      ## Package List
      packages = find_packages('src'),

      ###########################################
      ## Package Data
      package_data={'eWRT.stat.language': ['data/*.csv'],
                    'eWRT.input.stock': ['data/*.csv'],
      },
)
